@@ -7,13 +7,14 @@
 #include <memory>
 
 #include "base/bind.h"
-#include "base/bind_helpers.h"
-#include "base/macros.h"
+#include "base/callback_helpers.h"
 #include "base/memory/ref_counted.h"
 #include "base/sequence_token.h"
+#include "base/test/bind.h"
 #include "base/test/gtest_util.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/threading/simple_thread.h"
+#include "base/threading/thread_local.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -26,13 +27,14 @@ class RunCallbackThread : public SimpleThread {
   explicit RunCallbackThread(OnceClosure callback)
       : SimpleThread("RunCallbackThread"), callback_(std::move(callback)) {}
 
+  RunCallbackThread(const RunCallbackThread&) = delete;
+  RunCallbackThread& operator=(const RunCallbackThread&) = delete;
+
  private:
   // SimpleThread:
   void Run() override { std::move(callback_).Run(); }
 
   OnceClosure callback_;
-
-  DISALLOW_COPY_AND_ASSIGN(RunCallbackThread);
 };
 
 // Runs a callback on a new thread synchronously.
@@ -87,7 +89,7 @@ TEST(ThreadCheckerTest,
   {
     ScopedSetSequenceTokenForCurrentThread
         scoped_set_sequence_token_for_current_thread(sequence_token);
-    thread_checker.reset(new ThreadCheckerImpl);
+    thread_checker = std::make_unique<ThreadCheckerImpl>();
   }
 
   {
@@ -112,7 +114,7 @@ TEST(ThreadCheckerTest,
   {
     ScopedSetSequenceTokenForCurrentThread
         scoped_set_sequence_token_for_current_thread(SequenceToken::Create());
-    thread_checker.reset(new ThreadCheckerImpl);
+    thread_checker = std::make_unique<ThreadCheckerImpl>();
   }
 
   {
@@ -152,7 +154,7 @@ TEST(ThreadCheckerTest, DisallowedSameThreadDifferentSequence) {
   {
     ScopedSetSequenceTokenForCurrentThread
         scoped_set_sequence_token_for_current_thread(SequenceToken::Create());
-    thread_checker.reset(new ThreadCheckerImpl);
+    thread_checker = std::make_unique<ThreadCheckerImpl>();
   }
 
   {
@@ -194,6 +196,112 @@ TEST(ThreadCheckerTest, DetachFromThreadWithSequenceToken) {
   EXPECT_FALSE(thread_checker.CalledOnValidThread());
 }
 
+// Owns a ThreadCheckerImpl and asserts that CalledOnValidThread() is valid
+// in ~ThreadCheckerOwner.
+class ThreadCheckerOwner {
+ public:
+  explicit ThreadCheckerOwner(bool detach_from_thread) {
+    if (detach_from_thread)
+      checker_.DetachFromThread();
+  }
+
+  ThreadCheckerOwner(const ThreadCheckerOwner&) = delete;
+  ThreadCheckerOwner& operator=(const ThreadCheckerOwner&) = delete;
+
+  ~ThreadCheckerOwner() { EXPECT_TRUE(checker_.CalledOnValidThread()); }
+
+ private:
+  ThreadCheckerImpl checker_;
+};
+
+// Verifies ThreadCheckerImpl::CalledOnValidThread() returns true if called
+// during thread destruction.
+TEST(ThreadCheckerTest, CalledOnValidThreadFromThreadDestruction) {
+  ThreadLocalOwnedPointer<ThreadCheckerOwner> thread_local_owner;
+  RunCallbackOnNewThreadSynchronously(BindLambdaForTesting([&]() {
+    thread_local_owner.Set(std::make_unique<ThreadCheckerOwner>(false));
+  }));
+}
+
+// Variant of CalledOnValidThreadFromThreadDestruction that calls
+// ThreadCheckerImpl::DetachFromThread().
+TEST(ThreadCheckerTest, CalledOnValidThreadFromThreadDestructionDetached) {
+  ThreadLocalOwnedPointer<ThreadCheckerOwner> thread_local_owner;
+  RunCallbackOnNewThreadSynchronously(BindLambdaForTesting([&]() {
+    thread_local_owner.Set(std::make_unique<ThreadCheckerOwner>(true));
+  }));
+}
+
+TEST(ThreadCheckerTest, Move) {
+  ThreadCheckerImpl initial;
+  EXPECT_TRUE(initial.CalledOnValidThread());
+
+  ThreadCheckerImpl move_constructed(std::move(initial));
+  EXPECT_TRUE(move_constructed.CalledOnValidThread());
+
+  ThreadCheckerImpl move_assigned;
+  move_assigned = std::move(move_constructed);
+  EXPECT_TRUE(move_assigned.CalledOnValidThread());
+
+  // The two ThreadCheckerImpls moved from should be able to rebind to another
+  // thread.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectCalledOnValidThread, Unretained(&initial)));
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectCalledOnValidThread, Unretained(&move_constructed)));
+
+  // But the latest one shouldn't be able to run on another thread.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectNotCalledOnValidThread, Unretained(&move_assigned)));
+
+  EXPECT_TRUE(move_assigned.CalledOnValidThread());
+}
+
+TEST(ThreadCheckerTest, MoveAssignIntoDetached) {
+  ThreadCheckerImpl initial;
+
+  ThreadCheckerImpl move_assigned;
+  move_assigned.DetachFromThread();
+  move_assigned = std::move(initial);
+
+  // |initial| is detached after move.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectCalledOnValidThread, Unretained(&initial)));
+
+  // |move_assigned| should be associated with the main thread.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectNotCalledOnValidThread, Unretained(&move_assigned)));
+
+  EXPECT_TRUE(move_assigned.CalledOnValidThread());
+}
+
+TEST(ThreadCheckerTest, MoveFromDetachedRebinds) {
+  ThreadCheckerImpl initial;
+  initial.DetachFromThread();
+
+  ThreadCheckerImpl moved_into(std::move(initial));
+
+  // |initial| is still detached after move.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectCalledOnValidThread, Unretained(&initial)));
+
+  // |moved_into| is bound to the current thread as part of the move.
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectNotCalledOnValidThread, Unretained(&moved_into)));
+  EXPECT_TRUE(moved_into.CalledOnValidThread());
+}
+
+TEST(ThreadCheckerTest, MoveOffThreadBanned) {
+  testing::GTEST_FLAG(death_test_style) = "threadsafe";
+
+  ThreadCheckerImpl other_thread;
+  other_thread.DetachFromThread();
+  RunCallbackOnNewThreadSynchronously(
+      BindOnce(&ExpectCalledOnValidThread, Unretained(&other_thread)));
+
+  EXPECT_DCHECK_DEATH(ThreadCheckerImpl main_thread(std::move(other_thread)));
+}
+
 namespace {
 
 // This fixture is a helper for unit testing the thread checker macros as it is
@@ -205,37 +313,37 @@ class ThreadCheckerMacroTest : public testing::Test {
  public:
   ThreadCheckerMacroTest() = default;
 
+  ThreadCheckerMacroTest(const ThreadCheckerMacroTest&) = delete;
+  ThreadCheckerMacroTest& operator=(const ThreadCheckerMacroTest&) = delete;
+
   void ExpectDeathOnOtherThread() {
 #if DCHECK_IS_ON()
-    EXPECT_DCHECK_DEATH({ DCHECK_CALLED_ON_VALID_THREAD(my_thread_checker_); });
+    EXPECT_DCHECK_DEATH({ DCHECK_CALLED_ON_VALID_THREAD(thread_checker_); });
 #else
     // Happily no-ops on non-dcheck builds.
-    DCHECK_CALLED_ON_VALID_THREAD(my_thread_checker_);
+    DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 #endif
   }
 
   void ExpectNoDeathOnOtherThreadAfterDetach() {
-    DCHECK_CALLED_ON_VALID_THREAD(my_thread_checker_);
-    DCHECK_CALLED_ON_VALID_THREAD(my_thread_checker_)
-        << "Make sure it compiles when DCHECK is off";
+    DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   }
 
  protected:
-  THREAD_CHECKER(my_thread_checker_);
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ThreadCheckerMacroTest);
+  THREAD_CHECKER(thread_checker_);
 };
 
 }  // namespace
 
 TEST_F(ThreadCheckerMacroTest, Macros) {
+  testing::GTEST_FLAG(death_test_style) = "threadsafe";
+
   THREAD_CHECKER(my_thread_checker);
 
   RunCallbackOnNewThreadSynchronously(BindOnce(
       &ThreadCheckerMacroTest::ExpectDeathOnOtherThread, Unretained(this)));
 
-  DETACH_FROM_THREAD(my_thread_checker_);
+  DETACH_FROM_THREAD(thread_checker_);
 
   RunCallbackOnNewThreadSynchronously(
       BindOnce(&ThreadCheckerMacroTest::ExpectNoDeathOnOtherThreadAfterDetach,
