@@ -4,24 +4,30 @@
 
 #include "chrome/browser/ui/webui/chromeos/internet_detail_dialog.h"
 
-#include "ash/public/cpp/ash_features.h"
+#include "ash/constants/ash_features.h"
 #include "ash/public/cpp/network_config_service.h"
 #include "base/json/json_writer.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/ui/webui/chromeos/network_element_localized_strings_provider.h"
+#include "chrome/browser/ui/webui/chromeos/cellular_setup/cellular_setup_localized_strings_provider.h"
+#include "chrome/browser/ui/webui/webui_util.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/browser_resources.h"
 #include "chrome/grit/generated_resources.h"
+#include "chrome/grit/internet_detail_dialog_resources.h"
+#include "chrome/grit/internet_detail_dialog_resources_map.h"
 #include "chromeos/network/network_handler.h"
 #include "chromeos/network/network_state.h"
 #include "chromeos/network/network_state_handler.h"
 #include "chromeos/network/network_util.h"
 #include "chromeos/services/network_config/public/mojom/cros_network_config.mojom.h"  // nogncheck
+#include "chromeos/strings/grit/chromeos_strings.h"
 #include "components/strings/grit/components_strings.h"
 #include "content/public/browser/web_ui.h"
 #include "content/public/browser/web_ui_data_source.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/chromeos/strings/grit/ui_chromeos_strings.h"
+#include "ui/chromeos/strings/network_element_localized_strings_provider.h"
 
 namespace chromeos {
 
@@ -34,9 +40,9 @@ int s_internet_detail_dialog_count = 0;
 
 void AddInternetStrings(content::WebUIDataSource* html_source) {
   // Add default strings first.
-  chromeos::network_element::AddLocalizedStrings(html_source);
-  chromeos::network_element::AddOncLocalizedStrings(html_source);
-  chromeos::network_element::AddDetailsLocalizedStrings(html_source);
+  ui::network_element::AddLocalizedStrings(html_source);
+  ui::network_element::AddOncLocalizedStrings(html_source);
+  ui::network_element::AddDetailsLocalizedStrings(html_source);
   // Add additional strings and overrides needed by the dialog.
   struct {
     const char* name;
@@ -52,7 +58,7 @@ void AddInternetStrings(content::WebUIDataSource* html_source) {
       {"networkSectionProxy", IDS_SETTINGS_INTERNET_NETWORK_SECTION_PROXY},
       {"networkIPConfigAuto", IDS_SETTINGS_INTERNET_NETWORK_IP_CONFIG_AUTO},
       {"save", IDS_SAVE},
-      // Override for network_element::AddDetailsLocalizedStrings
+      // Override for ui::network_element::AddDetailsLocalizedStrings
       {"networkProxyConnectionType",
        IDS_SETTINGS_INTERNET_NETWORK_PROXY_CONNECTION_TYPE_DIALOG},
   };
@@ -60,12 +66,6 @@ void AddInternetStrings(content::WebUIDataSource* html_source) {
     html_source->AddLocalizedString(entry.name, entry.id);
 }
 
-base::string16 GetNetworkName16(const NetworkState& network) {
-  return network.Matches(NetworkTypePattern::Ethernet())
-             ? l10n_util::GetStringUTF16(IDS_NETWORK_TYPE_ETHERNET)
-             : base::UTF8ToUTF16(network.name());
-}
-
 std::string GetNetworkName8(const NetworkState& network) {
   return network.Matches(NetworkTypePattern::Ethernet())
              ? l10n_util::GetStringUTF8(IDS_NETWORK_TYPE_ETHERNET)
@@ -80,7 +80,8 @@ bool InternetDetailDialog::IsShown() {
 }
 
 // static
-void InternetDetailDialog::ShowDialog(const std::string& network_id) {
+void InternetDetailDialog::ShowDialog(const std::string& network_id,
+                                      gfx::NativeWindow parent) {
   auto* network_state_handler = NetworkHandler::Get()->network_state_handler();
   const NetworkState* network;
   if (!network_id.empty())
@@ -98,12 +99,12 @@ void InternetDetailDialog::ShowDialog(const std::string& network_id) {
   }
 
   InternetDetailDialog* dialog = new InternetDetailDialog(*network);
-  dialog->ShowSystemDialog();
+  dialog->ShowSystemDialog(parent);
 }
 
 InternetDetailDialog::InternetDetailDialog(const NetworkState& network)
     : SystemWebDialogDelegate(GURL(chrome::kChromeUIIntenetDetailDialogURL),
-                              GetNetworkName16(network)),
+                              /* title= */ std::u16string()),
       network_id_(network.guid()),
       network_type_(network_util::TranslateShillTypeToONC(network.type())),
       network_name_(GetNetworkName8(network)) {
@@ -139,19 +140,19 @@ InternetDetailDialogUI::InternetDetailDialogUI(content::WebUI* web_ui)
     : ui::MojoWebDialogUI(web_ui) {
   content::WebUIDataSource* source = content::WebUIDataSource::Create(
       chrome::kChromeUIInternetDetailDialogHost);
+  source->DisableTrustedTypesCSP();
   source->AddBoolean("showTechnologyBadge",
                      !ash::features::IsSeparateNetworkIconsEnabled());
+  cellular_setup::AddNonStringLoadTimeData(source);
   AddInternetStrings(source);
   source->AddLocalizedString("title", IDS_SETTINGS_INTERNET_DETAIL);
   source->UseStringsJs();
-#if BUILDFLAG(OPTIMIZE_WEBUI)
-  source->SetDefaultResource(IDR_INTERNET_DETAIL_DIALOG_VULCANIZED_HTML);
-  source->AddResourcePath("crisper.js", IDR_INTERNET_DETAIL_DIALOG_CRISPER_JS);
-#else
-  source->SetDefaultResource(IDR_INTERNET_DETAIL_DIALOG_HTML);
-  source->AddResourcePath("internet_detail_dialog.js",
-                          IDR_INTERNET_DETAIL_DIALOG_JS);
-#endif
+
+  webui::SetupWebUIDataSource(
+      source,
+      base::make_span(kInternetDetailDialogResources,
+                      kInternetDetailDialogResourcesSize),
+      IDR_INTERNET_DETAIL_DIALOG_INTERNET_DETAIL_DIALOG_CONTAINER_HTML);
   content::WebUIDataSource::Add(Profile::FromWebUI(web_ui), source);
 }
 
