@@ -6,6 +6,7 @@
 
 #include "base/bind.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/chromeos_buildflags.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_attributes_entry.h"
@@ -13,13 +14,11 @@
 #include "chrome/common/pref_names.h"
 #include "ui/base/webui/web_ui_util.h"
 
-#if defined(OS_CHROMEOS)
-#include "chrome/browser/chrome_notification_types.h"
-#include "chrome/browser/chromeos/profiles/profile_helper.h"
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "chrome/browser/ash/profiles/profile_helper.h"
 #include "chrome/browser/ui/webui/chromeos/user_image_source.h"
 #include "components/account_id/account_id.h"
 #include "components/user_manager/user_manager.h"
-#include "content/public/browser/notification_service.h"
 #else
 #include "chrome/browser/profiles/profile_avatar_icon_util.h"
 #include "chrome/browser/profiles/profile_statistics.h"
@@ -35,13 +34,8 @@ const char ProfileInfoHandler::kProfileInfoChangedEventName[] =
 const char ProfileInfoHandler::kProfileStatsCountReadyEventName[] =
     "profile-stats-count-ready";
 
-ProfileInfoHandler::ProfileInfoHandler(Profile* profile)
-    : profile_(profile),
-#if defined(OS_CHROMEOS)
-      user_manager_observer_(this),
-#endif
-      profile_observer_(this) {
-#if defined(OS_CHROMEOS)
+ProfileInfoHandler::ProfileInfoHandler(Profile* profile) : profile_(profile) {
+#if BUILDFLAG(IS_CHROMEOS_ASH)
   // Set up the chrome://userimage/ source.
   content::URLDataSource::Add(profile,
                               std::make_unique<chromeos::UserImageSource>());
@@ -55,7 +49,7 @@ void ProfileInfoHandler::RegisterMessages() {
       "getProfileInfo",
       base::BindRepeating(&ProfileInfoHandler::HandleGetProfileInfo,
                           base::Unretained(this)));
-#if !defined(OS_CHROMEOS)
+#if !BUILDFLAG(IS_CHROMEOS_ASH)
   web_ui()->RegisterMessageCallback(
       "getProfileStatsCount",
       base::BindRepeating(&ProfileInfoHandler::HandleGetProfileStats,
@@ -64,26 +58,29 @@ void ProfileInfoHandler::RegisterMessages() {
 }
 
 void ProfileInfoHandler::OnJavascriptAllowed() {
-  profile_observer_.Add(
+  profile_observation_.Observe(
       &g_browser_process->profile_manager()->GetProfileAttributesStorage());
 
-#if defined(OS_CHROMEOS)
-  user_manager_observer_.Add(user_manager::UserManager::Get());
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  user_manager_observation_.Observe(user_manager::UserManager::Get());
 #endif
 }
 
 void ProfileInfoHandler::OnJavascriptDisallowed() {
   callback_weak_ptr_factory_.InvalidateWeakPtrs();
 
-  profile_observer_.Remove(
-      &g_browser_process->profile_manager()->GetProfileAttributesStorage());
+  DCHECK(profile_observation_.IsObservingSource(
+      &g_browser_process->profile_manager()->GetProfileAttributesStorage()));
+  profile_observation_.Reset();
 
-#if defined(OS_CHROMEOS)
-  user_manager_observer_.Remove(user_manager::UserManager::Get());
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  DCHECK(user_manager_observation_.IsObservingSource(
+      user_manager::UserManager::Get()));
+  user_manager_observation_.Reset();
 #endif
 }
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 void ProfileInfoHandler::OnUserImageChanged(const user_manager::User& user) {
   PushProfileInfo();
 }
@@ -91,7 +88,7 @@ void ProfileInfoHandler::OnUserImageChanged(const user_manager::User& user) {
 
 void ProfileInfoHandler::OnProfileNameChanged(
     const base::FilePath& /* profile_path */,
-    const base::string16& /* old_profile_name */) {
+    const std::u16string& /* old_profile_name */) {
   PushProfileInfo();
 }
 
@@ -100,23 +97,22 @@ void ProfileInfoHandler::OnProfileAvatarChanged(
   PushProfileInfo();
 }
 
-void ProfileInfoHandler::HandleGetProfileInfo(const base::ListValue* args) {
+void ProfileInfoHandler::HandleGetProfileInfo(const base::Value::List& args) {
   AllowJavascript();
 
-  CHECK_EQ(1U, args->GetSize());
-  const base::Value* callback_id;
-  CHECK(args->Get(0, &callback_id));
+  CHECK_EQ(1U, args.size());
+  const base::Value& callback_id = args[0];
 
-  ResolveJavascriptCallback(*callback_id, *GetAccountNameAndIcon());
+  ResolveJavascriptCallback(callback_id, *GetAccountNameAndIcon());
 }
 
-#if !defined(OS_CHROMEOS)
-void ProfileInfoHandler::HandleGetProfileStats(const base::ListValue* args) {
+#if !BUILDFLAG(IS_CHROMEOS_ASH)
+void ProfileInfoHandler::HandleGetProfileStats(const base::Value::List& args) {
   AllowJavascript();
 
   ProfileStatisticsFactory::GetForProfile(profile_)->GatherStatistics(
-      base::Bind(&ProfileInfoHandler::PushProfileStatsCount,
-                 callback_weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&ProfileInfoHandler::PushProfileStatsCount,
+                          callback_weak_ptr_factory_.GetWeakPtr()));
 }
 
 void ProfileInfoHandler::PushProfileStatsCount(
@@ -137,13 +133,13 @@ void ProfileInfoHandler::PushProfileInfo() {
 }
 
 std::unique_ptr<base::DictionaryValue>
-ProfileInfoHandler::GetAccountNameAndIcon() const {
+ProfileInfoHandler::GetAccountNameAndIcon() {
   std::string name;
   std::string icon_url;
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
   const user_manager::User* user =
-      chromeos::ProfileHelper::Get()->GetUserByProfile(profile_);
+      ash::ProfileHelper::Get()->GetUserByProfile(profile_);
   DCHECK(user);
   name = base::UTF16ToUTF8(user->GetDisplayName());
 
@@ -152,12 +148,13 @@ ProfileInfoHandler::GetAccountNameAndIcon() const {
   scoped_refptr<base::RefCountedMemory> image =
       chromeos::UserImageSource::GetUserImage(user->GetAccountId());
   icon_url = webui::GetPngDataUrl(image->front(), image->size());
-#else   // !defined(OS_CHROMEOS)
-  ProfileAttributesEntry* entry;
-  if (g_browser_process->profile_manager()
+#else   // !BUILDFLAG(IS_CHROMEOS_ASH)
+  ProfileAttributesEntry* entry =
+      g_browser_process->profile_manager()
           ->GetProfileAttributesStorage()
-          .GetProfileAttributesWithPath(profile_->GetPath(), &entry)) {
-    name = base::UTF16ToUTF8(entry->GetName());
+          .GetProfileAttributesWithPath(profile_->GetPath());
+  if (entry) {
+    name = base::UTF16ToUTF8(entry->GetLocalProfileName());
     // TODO(crbug.com/710660): return chrome://theme/IDR_PROFILE_AVATAR_*
     // and update theme_source.cc to get high res avatar icons. This does less
     // work here, sends less over IPC, and is more stable with returned results.
@@ -166,11 +163,11 @@ ProfileInfoHandler::GetAccountNameAndIcon() const {
         entry->GetAvatarIcon(), true, kAvatarIconSize, kAvatarIconSize);
     icon_url = webui::GetBitmapDataUrl(icon.AsBitmap());
   }
-#endif  // defined(OS_CHROMEOS)
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
   auto response = std::make_unique<base::DictionaryValue>();
-  response->SetString("name", name);
-  response->SetString("iconUrl", icon_url);
+  response->SetStringKey("name", name);
+  response->SetStringKey("iconUrl", icon_url);
   return response;
 }
 
