@@ -5,6 +5,7 @@
 // Custom binding for the fileManagerPrivate API.
 
 // Natives
+var blobNatives = requireNative('blob_natives');
 var fileManagerPrivateNatives = requireNative('file_manager_private');
 
 // Internals
@@ -28,7 +29,7 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
   var apiFunctions = bindingsAPI.apiFunctions;
 
   apiFunctions.setCustomCallback('searchDrive',
-      function(name, request, callback, response) {
+      function(callback, response) {
     if (response && !response.error && response.entries) {
       response.entries = response.entries.map(function(entry) {
         return GetExternalFileEntry(entry);
@@ -44,7 +45,7 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
   });
 
   apiFunctions.setCustomCallback('searchDriveMetadata',
-      function(name, request, callback, response) {
+      function(callback, response) {
     if (response && !response.error) {
       for (var i = 0; i < response.length; i++) {
         response[i].entry =
@@ -73,6 +74,12 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
     });
   });
 
+  apiFunctions.setHandleRequest('getVolumeRoot', function(options, callback) {
+    fileManagerPrivateInternal.getVolumeRoot(options, function(entry) {
+      callback(entry ? GetExternalFileEntry(entry) : undefined);
+    });
+  });
+
   apiFunctions.setHandleRequest('getEntryProperties',
                                 function(entries, names, callback) {
     var urls = entries.map(function(entry) {
@@ -117,32 +124,84 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
     fileManagerPrivateInternal.getMimeType(url, callback);
   });
 
+  apiFunctions.setHandleRequest('getContentMimeType',
+      function(fileEntry, callback) {
+    fileEntry.file(blob => {
+      var blobUUID = blobNatives.GetBlobUuid(blob);
+
+      if (!blob || !blob.size) {
+        callback(undefined);
+        return;
+      }
+
+      var onGetContentMimeType = function(blob, mimeType) {
+        callback(mimeType ? mimeType : undefined);
+      }.bind(this, blob);  // Bind a blob reference: crbug.com/415792#c12
+
+      fileManagerPrivateInternal.getContentMimeType(
+          blobUUID, onGetContentMimeType);
+    }, (error) => {
+      var errorUUID = '';
+
+      var onGetContentMimeType = function() {
+        chrome.runtime.lastError.DOMError = /** @type {!DOMError} */ (error);
+        callback(undefined);
+      }.bind(this);
+
+      fileManagerPrivateInternal.getContentMimeType(
+          errorUUID, onGetContentMimeType);
+    });
+  });
+
+  apiFunctions.setHandleRequest('getContentMetadata',
+      function(fileEntry, mimeType, includeImages, callback) {
+    fileEntry.file(blob => {
+      var blobUUID = blobNatives.GetBlobUuid(blob);
+
+      if (!blob || !blob.size) {
+        callback(undefined);
+        return;
+      }
+
+      var onGetContentMetadata = function(blob, metadata) {
+        callback(metadata ? metadata : undefined);
+      }.bind(this, blob);  // Bind a blob reference: crbug.com/415792#c12
+
+      fileManagerPrivateInternal.getContentMetadata(
+          blobUUID, mimeType, !!includeImages, onGetContentMetadata);
+    }, (error) => {
+      var errorUUID = '';
+
+      var onGetContentMetadata = function() {
+        chrome.runtime.lastError.DOMError = /** @type {!DOMError} */ (error);
+        callback(undefined);
+      }.bind(this);
+
+      fileManagerPrivateInternal.getContentMetadata(
+          errorUUID, mimeType, false, onGetContentMetadata);
+    });
+  });
+
   apiFunctions.setHandleRequest('pinDriveFile', function(entry, pin, callback) {
     var url = getEntryURL(entry);
     fileManagerPrivateInternal.pinDriveFile(url, pin, callback);
   });
 
-  apiFunctions.setHandleRequest(
-      'ensureFileDownloaded', function(entry, callback) {
-        var url = getEntryURL(entry);
-        fileManagerPrivateInternal.ensureFileDownloaded(url, callback);
-      });
-
   apiFunctions.setHandleRequest('executeTask',
-      function(taskId, entries, callback) {
+      function(descriptor, entries, callback) {
         var urls = entries.map(function(entry) {
           return getEntryURL(entry);
         });
-        fileManagerPrivateInternal.executeTask(taskId, urls, callback);
+        fileManagerPrivateInternal.executeTask(descriptor, urls, callback);
       });
 
   apiFunctions.setHandleRequest('setDefaultTask',
-      function(taskId, entries, mimeTypes, callback) {
+      function(descriptor, entries, mimeTypes, callback) {
         var urls = entries.map(function(entry) {
           return getEntryURL(entry);
         });
         fileManagerPrivateInternal.setDefaultTask(
-            taskId, urls, mimeTypes, callback);
+            descriptor, urls, mimeTypes, callback);
       });
 
   apiFunctions.setHandleRequest('getFileTasks', function(entries, callback) {
@@ -157,26 +216,13 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
     fileManagerPrivateInternal.getDownloadUrl(url, callback);
   });
 
-  apiFunctions.setHandleRequest('requestDriveShare', function(
-        entry, shareType, callback) {
-    var url = getEntryURL(entry);
-    fileManagerPrivateInternal.requestDriveShare(url, shareType, callback);
-  });
-
-  apiFunctions.setHandleRequest('setEntryTag', function(
-        entry, visibility, key, value, callback) {
-    var url = getEntryURL(entry);
-    fileManagerPrivateInternal.setEntryTag(
-        url, visibility, key, value, callback);
-  });
-
-  apiFunctions.setHandleRequest('cancelFileTransfers', function(
-        entries, callback) {
-    var urls = entries.map(function(entry) {
-      return getEntryURL(entry);
-    });
-    fileManagerPrivateInternal.cancelFileTransfers(urls, callback);
-  });
+  apiFunctions.setHandleRequest(
+      'getDisallowedTransfers', function(entries, destinationEntry, callback) {
+        var sourceUrls = entries.map(getEntryURL);
+        var destinationUrl = getEntryURL(destinationEntry);
+        fileManagerPrivateInternal.getDisallowedTransfers(
+            sourceUrls, destinationUrl, callback);
+      });
 
   apiFunctions.setHandleRequest('startCopy', function(
         entry, parentEntry, newName, callback) {
@@ -186,15 +232,12 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
         url, parentUrl, newName, callback);
   });
 
-  apiFunctions.setHandleRequest('zipSelection', function(
-        parentEntry, entries, destName, callback) {
-    var parentUrl = getEntryURL(parentEntry);
-    var urls = entries.map(function(entry) {
-      return getEntryURL(entry);
-    });
-    fileManagerPrivateInternal.zipSelection(
-        parentUrl, urls, destName, callback);
-  });
+  apiFunctions.setHandleRequest(
+      'zipSelection',
+      (entries, parentEntry, destName, callback) =>
+          fileManagerPrivateInternal.zipSelection(
+              getEntryURL(parentEntry), entries.map(getEntryURL), destName,
+              callback));
 
   apiFunctions.setHandleRequest('validatePathNameLength', function(
         entry, name, callback) {
@@ -210,8 +253,8 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
   });
 
   apiFunctions.setHandleRequest('getRecentFiles', function(
-        restriction, callback) {
-    fileManagerPrivateInternal.getRecentFiles(restriction, function(
+        restriction, file_type, callback) {
+    fileManagerPrivateInternal.getRecentFiles(restriction, file_type, function(
           entryDescriptions) {
       callback(entryDescriptions.map(function(description) {
         return GetExternalFileEntry(description);
@@ -258,14 +301,27 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
     fileManagerPrivateInternal.installLinuxPackage(url, callback);
   });
 
-  apiFunctions.setHandleRequest('getThumbnail', function(
+  apiFunctions.setHandleRequest('getDriveThumbnail', function(
         entry, cropToSquare, callback) {
     var url = getEntryURL(entry);
-    fileManagerPrivateInternal.getThumbnail(url, cropToSquare, callback);
+    fileManagerPrivateInternal.getDriveThumbnail(url, cropToSquare, callback);
+  });
+
+  apiFunctions.setHandleRequest('getPdfThumbnail', function(
+        entry, width, height, callback) {
+    var url = getEntryURL(entry);
+    fileManagerPrivateInternal.getPdfThumbnail(url, width, height, callback);
+  });
+
+  apiFunctions.setHandleRequest('getArcDocumentsProviderThumbnail', function(
+        entry, widthHint, heightHint, callback) {
+    var url = getEntryURL(entry);
+    fileManagerPrivateInternal.getArcDocumentsProviderThumbnail(
+        url, widthHint, heightHint, callback);
   });
 
   apiFunctions.setCustomCallback('searchFiles',
-      function(name, request, callback, response) {
+      function(callback, response) {
     if (response && !response.error && response.entries) {
       response.entries = response.entries.map(function(entry) {
         return GetExternalFileEntry(entry);
@@ -282,6 +338,43 @@ apiBridge.registerCustomHook(function(bindingsAPI) {
     }
   });
 
+  apiFunctions.setHandleRequest('importCrostiniImage', function(entry) {
+    const url = getEntryURL(entry);
+    fileManagerPrivateInternal.importCrostiniImage(url);
+  });
+
+  apiFunctions.setHandleRequest(
+      'sharesheetHasTargets', function(entries, callback) {
+        var urls = entries.map(function(entry) {
+          return getEntryURL(entry);
+        });
+        fileManagerPrivateInternal.sharesheetHasTargets(urls, callback);
+      });
+
+  apiFunctions.setHandleRequest(
+      'invokeSharesheet', function(entries, launchSource, callback) {
+        var urls = entries.map(function(entry) {
+          return getEntryURL(entry);
+        });
+        fileManagerPrivateInternal.invokeSharesheet(
+            urls, launchSource, callback);
+      });
+
+  apiFunctions.setHandleRequest(
+      'toggleAddedToHoldingSpace', function(entries, added, callback) {
+        const urls = entries.map(entry => getEntryURL(entry));
+        fileManagerPrivateInternal.toggleAddedToHoldingSpace(
+            urls, added, callback);
+      });
+
+  apiFunctions.setHandleRequest('startIOTask', function(type, entries, params) {
+    const urls = entries.map(entry => getEntryURL(entry));
+    let newParams = {};
+    if (params.destinationFolder) {
+      newParams.destinationFolderUrl = getEntryURL(params.destinationFolder);
+    }
+    fileManagerPrivateInternal.startIOTask(type, urls, newParams);
+  });
 });
 
 bindingUtil.registerEventArgumentMassager(
