@@ -4,11 +4,11 @@
 
 #include "components/autofill/core/browser/webdata/autofill_profile_sync_difference_tracker.h"
 
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/mock_callback.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
 #include "base/time/time.h"
 #include "components/autofill/core/browser/autofill_profile_sync_util.h"
 #include "components/autofill/core/browser/data_model/autofill_profile.h"
@@ -30,18 +30,29 @@ using testing::ElementsAre;
 using testing::IsEmpty;
 
 // Some guids for testing.
-const char kGuidA[] = "EDC609ED-7EEE-4F27-B00C-423242A9C44A";
-const char kGuidB[] = "EDC609ED-7EEE-4F27-B00C-423242A9C44B";
+const char kSmallerGuid[] = "EDC609ED-7EEE-4F27-B00C-423242A9C44A";
+const char kBiggerGuid[] = "EDC609ED-7EEE-4F27-B00C-423242A9C44B";
 const char kHttpOrigin[] = "http://www.example.com/";
 const char kHttpsOrigin[] = "https://www.example.com/";
 const char kLocaleString[] = "en-US";
 const base::Time kJune2017 = base::Time::FromDoubleT(1497552271);
 
+struct UpdatesToSync {
+  std::vector<AutofillProfile> profiles_to_upload_to_sync;
+  std::vector<std::string> profiles_to_delete_from_sync;
+};
+
 }  // namespace
 
 class AutofillProfileSyncDifferenceTrackerTestBase : public testing::Test {
  public:
   AutofillProfileSyncDifferenceTrackerTestBase() {}
+
+  AutofillProfileSyncDifferenceTrackerTestBase(
+      const AutofillProfileSyncDifferenceTrackerTestBase&) = delete;
+  AutofillProfileSyncDifferenceTrackerTestBase& operator=(
+      const AutofillProfileSyncDifferenceTrackerTestBase&) = delete;
+
   ~AutofillProfileSyncDifferenceTrackerTestBase() override {}
 
   void SetUp() override {
@@ -61,28 +72,30 @@ class AutofillProfileSyncDifferenceTrackerTestBase : public testing::Test {
   }
 
   void IncorporateRemoteProfile(const AutofillProfile& profile) {
-    EXPECT_EQ(base::nullopt, tracker()->IncorporateRemoteProfile(
+    EXPECT_EQ(absl::nullopt, tracker()->IncorporateRemoteProfile(
                                  std::make_unique<AutofillProfile>(profile)));
   }
 
-  std::vector<AutofillProfile> FlushAndReturnProfilesToUploadToSync() {
-    EXPECT_EQ(base::nullopt,
+  UpdatesToSync FlushToSync() {
+    EXPECT_EQ(absl::nullopt,
               tracker()->FlushToLocal(
                   /*autofill_changes_callback=*/base::DoNothing()));
 
+    UpdatesToSync updates;
     std::vector<std::unique_ptr<AutofillProfile>> vector_of_unique_ptrs;
-    EXPECT_EQ(base::nullopt,
+    EXPECT_EQ(absl::nullopt,
               tracker()->FlushToSync(
-                  /*profiles_to_upload_to_sync=*/&vector_of_unique_ptrs));
+                  /*profiles_to_upload_to_sync=*/&vector_of_unique_ptrs,
+                  /*profiles_to_delete_from_sync=*/&updates
+                      .profiles_to_delete_from_sync));
 
     // Copy all the elements by value so that we have a vector that is easier to
     // work with in the test.
-    std::vector<AutofillProfile> vector_of_values;
     for (const std::unique_ptr<AutofillProfile>& entry :
          vector_of_unique_ptrs) {
-      vector_of_values.push_back(*entry);
+      updates.profiles_to_upload_to_sync.push_back(*entry);
     }
-    return vector_of_values;
+    return updates;
   }
 
   std::vector<AutofillProfile> GetAllLocalData() {
@@ -107,131 +120,160 @@ class AutofillProfileSyncDifferenceTrackerTestBase : public testing::Test {
  private:
   autofill::TestAutofillClock test_clock_;
   base::ScopedTempDir temp_dir_;
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   AutofillTable table_;
   WebDatabase db_;
-
-  DISALLOW_COPY_AND_ASSIGN(AutofillProfileSyncDifferenceTrackerTestBase);
 };
 
 class AutofillProfileSyncDifferenceTrackerTest
     : public AutofillProfileSyncDifferenceTrackerTestBase {
  public:
   AutofillProfileSyncDifferenceTrackerTest() : tracker_(table()) {}
+
+  AutofillProfileSyncDifferenceTrackerTest(
+      const AutofillProfileSyncDifferenceTrackerTest&) = delete;
+  AutofillProfileSyncDifferenceTrackerTest& operator=(
+      const AutofillProfileSyncDifferenceTrackerTest&) = delete;
+
   ~AutofillProfileSyncDifferenceTrackerTest() override {}
 
   AutofillProfileSyncDifferenceTracker* tracker() override { return &tracker_; }
 
  private:
   AutofillProfileSyncDifferenceTracker tracker_;
-
-  DISALLOW_COPY_AND_ASSIGN(AutofillProfileSyncDifferenceTrackerTest);
 };
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        IncorporateRemoteProfileShouldOverwriteProfileWithSameKey) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(NAME_FIRST, u"John");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile is completely different but it has the same key.
-  AutofillProfile remote = AutofillProfile(kGuidA, kHttpsOrigin);
-  remote.SetRawInfo(NAME_FIRST, ASCIIToUTF16("Tom"));
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  remote.SetRawInfo(NAME_FIRST, u"Tom");
+  remote.FinalizeAfterImport();
 
   IncorporateRemoteProfile(remote);
 
   // Nothing gets uploaded to sync and the remote profile wins.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(remote));
 }
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        IncorporateRemoteProfileShouldOverwriteUnverifiedProfileByVerified) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpsOrigin);
-  local.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  local.SetRawInfo(NAME_FIRST, u"John");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile has the same key but it is not verified.
-  AutofillProfile remote = AutofillProfile(kGuidA, kSettingsOrigin);
-  remote.SetRawInfo(NAME_FIRST, ASCIIToUTF16("Tom"));
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kSettingsOrigin);
+  remote.SetRawInfo(NAME_FIRST, u"Tom");
+  remote.FinalizeAfterImport();
 
   IncorporateRemoteProfile(remote);
 
   // Nothing gets uploaded to sync and the local profile wins.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(remote));
 }
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        IncorporateRemoteProfileShouldNotOverwriteVerifiedProfileByUnverified) {
-  AutofillProfile local = AutofillProfile(kGuidA, kSettingsOrigin);
-  local.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kSettingsOrigin);
+  local.SetRawInfo(NAME_FIRST, u"John");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile has the same key but it is not verified.
-  AutofillProfile remote = AutofillProfile(kGuidA, kHttpsOrigin);
-  remote.SetRawInfo(NAME_FIRST, ASCIIToUTF16("Tom"));
-
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  remote.SetRawInfo(NAME_FIRST, u"Tom");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
 
   // Nothing gets uploaded to sync and the local profile wins.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(local));
 }
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        IncorporateRemoteProfileShouldNotOverwriteFullNameByEmptyString) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(NAME_FULL, ASCIIToUTF16("John"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(NAME_FULL, u"John");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile has the same key.
-  AutofillProfile remote = AutofillProfile(kGuidA, kHttpsOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("2 2st st"));
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"2 2st st");
 
   AutofillProfile merged(remote);
-  merged.SetRawInfo(NAME_FULL, ASCIIToUTF16("John"));
-
+  merged.SetRawInfo(NAME_FULL, u"John");
+  merged.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
 
   // Nothing gets uploaded to sync and the remote profile wins except for the
   // full name.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(merged));
 }
 
-TEST_F(AutofillProfileSyncDifferenceTrackerTest,
-       IncorporateRemoteProfileShouldMergeIdenticalProfilesWithDifferentKeys) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+TEST_F(
+    AutofillProfileSyncDifferenceTrackerTest,
+    IncorporateRemoteProfileShouldKeepRemoteKeyWhenMergingDuplicateProfileWithBiggerKey) {
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(NAME_FIRST, u"John");
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile is identical to the local one, except that the guids and
   // origins are different.
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpsOrigin);
-  remote.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpsOrigin);
+  remote.SetRawInfo(NAME_FIRST, u"John");
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
 
   // Nothing gets uploaded to sync and the remote profile wins.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync,
+              ElementsAre(std::string(kSmallerGuid)));
   EXPECT_THAT(GetAllLocalData(), ElementsAre(remote));
 }
 
 TEST_F(
     AutofillProfileSyncDifferenceTrackerTest,
-    IncorporateRemoteProfileShouldMergeIdenticalProfilesWithDifferentKeysButKeepVerifiedOrigin) {
-  AutofillProfile local = AutofillProfile(kGuidA, kSettingsOrigin);
-  local.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+    IncorporateRemoteProfileShouldKeepRemoteKeyAndLocalOriginWhenMergingDuplicateProfileWithBiggerKey) {
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kSettingsOrigin);
+  local.SetRawInfoWithVerificationStatus(
+      NAME_FIRST, u"John", structured_address::VerificationStatus::kObserved);
+  local.SetRawInfoWithVerificationStatus(
+      ADDRESS_HOME_STREET_ADDRESS, u"1 1st st",
+      structured_address::VerificationStatus::kObserved);
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile has the same key.
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpsOrigin);
-  remote.SetRawInfo(NAME_FIRST, ASCIIToUTF16("John"));
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpsOrigin);
+  remote.SetRawInfoWithVerificationStatus(
+      NAME_FIRST, u"John", structured_address::VerificationStatus::kObserved);
+  remote.SetRawInfoWithVerificationStatus(
+      ADDRESS_HOME_STREET_ADDRESS, u"1 1st st",
+      structured_address::VerificationStatus::kObserved);
+  remote.FinalizeAfterImport();
 
   AutofillProfile merged(remote);
   merged.set_origin(kSettingsOrigin);
@@ -240,7 +282,71 @@ TEST_F(
 
   // Nothing gets uploaded to sync and the remote profile wins except for the
   // full name.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), ElementsAre(merged));
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(merged));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync,
+              ElementsAre(std::string(kSmallerGuid)));
+  EXPECT_THAT(GetAllLocalData(), ElementsAre(merged));
+}
+
+TEST_F(
+    AutofillProfileSyncDifferenceTrackerTest,
+    IncorporateRemoteProfileShouldKeepLocalKeyWhenMergingDuplicateProfileWithSmallerKey) {
+  AutofillProfile local = AutofillProfile(kBiggerGuid, kHttpOrigin);
+  local.SetRawInfo(NAME_FIRST, u"John");
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  local.FinalizeAfterImport();
+  AddAutofillProfilesToTable({local});
+
+  // The remote profile is identical to the local one, except that the guids and
+  // origins are different.
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  remote.SetRawInfo(NAME_FIRST, u"John");
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.FinalizeAfterImport();
+  IncorporateRemoteProfile(remote);
+
+  // Nothing gets uploaded to sync and the remote profile wins.
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync,
+              ElementsAre(std::string(kSmallerGuid)));
+  EXPECT_THAT(GetAllLocalData(), ElementsAre(local));
+}
+
+TEST_F(
+    AutofillProfileSyncDifferenceTrackerTest,
+    IncorporateRemoteProfileShouldKeepLocalKeyAndRemoteOriginWhenMergingDuplicateProfileWithSmallerKey) {
+  AutofillProfile local = AutofillProfile(kBiggerGuid, kHttpsOrigin);
+  local.SetRawInfoWithVerificationStatus(
+      NAME_FIRST, u"John",
+      structured_address::VerificationStatus::kUserVerified);
+  local.SetRawInfoWithVerificationStatus(
+      ADDRESS_HOME_STREET_ADDRESS, u"1 1st st",
+      structured_address::VerificationStatus::kUserVerified);
+  local.FinalizeAfterImport();
+  AddAutofillProfilesToTable({local});
+
+  // The remote profile has the same key.
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kSettingsOrigin);
+  remote.SetRawInfoWithVerificationStatus(
+      NAME_FIRST, u"John",
+      structured_address::VerificationStatus::kUserVerified);
+  remote.SetRawInfoWithVerificationStatus(
+      ADDRESS_HOME_STREET_ADDRESS, u"1 1st st",
+      structured_address::VerificationStatus::kUserVerified);
+  remote.FinalizeAfterImport();
+
+  AutofillProfile merged(local);
+  merged.set_origin(kSettingsOrigin);
+  IncorporateRemoteProfile(remote);
+
+  // Nothing gets uploaded to sync and the remote profile wins except for the
+  // full name.
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(merged));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync,
+              ElementsAre(std::string(kSmallerGuid)));
   EXPECT_THAT(GetAllLocalData(), ElementsAre(merged));
 }
 
@@ -249,20 +355,20 @@ TEST_F(AutofillProfileSyncDifferenceTrackerTest,
   MockCallback<base::OnceClosure> autofill_changes_callback;
 
   EXPECT_CALL(autofill_changes_callback, Run()).Times(0);
-  EXPECT_EQ(base::nullopt,
+  EXPECT_EQ(absl::nullopt,
             tracker()->FlushToLocal(autofill_changes_callback.Get()));
 }
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        FlushToLocalShouldCallbackWhenProfileDeleted) {
-  AutofillProfile local = AutofillProfile(kGuidA, kSettingsOrigin);
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kSettingsOrigin);
   AddAutofillProfilesToTable({local});
 
-  tracker()->IncorporateRemoteDelete(kGuidA);
+  tracker()->IncorporateRemoteDelete(kSmallerGuid);
 
   MockCallback<base::OnceClosure> autofill_changes_callback;
   EXPECT_CALL(autofill_changes_callback, Run()).Times(1);
-  EXPECT_EQ(base::nullopt,
+  EXPECT_EQ(absl::nullopt,
             tracker()->FlushToLocal(autofill_changes_callback.Get()));
 
   // On top of that, the profile should also get deleted.
@@ -271,12 +377,12 @@ TEST_F(AutofillProfileSyncDifferenceTrackerTest,
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        FlushToLocalShouldCallbackWhenProfileAdded) {
-  AutofillProfile remote = AutofillProfile(kGuidA, kSettingsOrigin);
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kSettingsOrigin);
   IncorporateRemoteProfile(remote);
 
   MockCallback<base::OnceClosure> autofill_changes_callback;
   EXPECT_CALL(autofill_changes_callback, Run()).Times(1);
-  EXPECT_EQ(base::nullopt,
+  EXPECT_EQ(absl::nullopt,
             tracker()->FlushToLocal(autofill_changes_callback.Get()));
 
   // On top of that, the profile should also get added.
@@ -285,19 +391,20 @@ TEST_F(AutofillProfileSyncDifferenceTrackerTest,
 
 TEST_F(AutofillProfileSyncDifferenceTrackerTest,
        FlushToLocalShouldCallbackWhenProfileUpdated) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpsOrigin);
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpsOrigin);
   AddAutofillProfilesToTable({local});
 
-  AutofillProfile remote = AutofillProfile(kGuidA, kHttpsOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile remote = AutofillProfile(kSmallerGuid, kHttpsOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
 
   MockCallback<base::OnceClosure> autofill_changes_callback;
   EXPECT_CALL(autofill_changes_callback, Run()).Times(1);
-  EXPECT_EQ(base::nullopt,
+  EXPECT_EQ(absl::nullopt,
             tracker()->FlushToLocal(autofill_changes_callback.Get()));
 
-  // On top of that, the profile with key kGuidA should also get updated.
+  // On top of that, the profile with key kSmallerGuid should also get updated.
   EXPECT_THAT(GetAllLocalData(), ElementsAre(remote));
 }
 
@@ -306,6 +413,12 @@ class AutofillProfileInitialSyncDifferenceTrackerTest
  public:
   AutofillProfileInitialSyncDifferenceTrackerTest()
       : initial_tracker_(table()) {}
+
+  AutofillProfileInitialSyncDifferenceTrackerTest(
+      const AutofillProfileInitialSyncDifferenceTrackerTest&) = delete;
+  AutofillProfileInitialSyncDifferenceTrackerTest& operator=(
+      const AutofillProfileInitialSyncDifferenceTrackerTest&) = delete;
+
   ~AutofillProfileInitialSyncDifferenceTrackerTest() override {}
 
   void MergeSimilarEntriesForInitialSync() {
@@ -318,23 +431,22 @@ class AutofillProfileInitialSyncDifferenceTrackerTest
 
  private:
   AutofillProfileInitialSyncDifferenceTracker initial_tracker_;
-
-  DISALLOW_COPY_AND_ASSIGN(AutofillProfileInitialSyncDifferenceTrackerTest);
 };
 
 TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
        MergeSimilarEntriesForInitialSyncShouldSyncUpChanges) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
   local.set_use_count(27);
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile matches the local one (except for origin and use count).
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpsOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-  remote.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpsOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
   remote.set_use_count(13);
-
+  remote.FinalizeAfterImport();
   // The remote profile wins (as regards the storage key).
   AutofillProfile merged(remote);
   // The local origin wins when merging.
@@ -346,92 +458,106 @@ TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
   MergeSimilarEntriesForInitialSync();
 
   // The merged profile needs to get uploaded back to sync and stored locally.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), ElementsAre(merged));
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(merged));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(merged));
 }
 
 TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
        MergeSimilarEntriesForInitialSyncShouldNotSyncUpWhenNotNeeded) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
   local.set_use_count(13);
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile matches the local one and has some additional data.
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-  remote.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
   // Merging two profile takes their max use count, so use count of 27 is taken.
   remote.set_use_count(27);
-
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
   MergeSimilarEntriesForInitialSync();
 
   // Nothing gets uploaded to sync and the remote profile wins.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), IsEmpty());
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, IsEmpty());
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(remote));
 }
 
 TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
        MergeSimilarEntriesForInitialSyncNotMatchNonsimilarEntries) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-  local.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  local.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile has a different street address.
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("2 2st st"));
-  remote.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
-
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"2 2st st");
+  remote.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
   MergeSimilarEntriesForInitialSync();
 
   // The local profile gets uploaded (due to initial sync) and the remote
   // profile gets stored locally.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), ElementsAre(local));
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(local));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(local, remote));
 }
 
 TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
        MergeSimilarEntriesForInitialSyncDoesNotMatchLocalVerifiedEntry) {
   // The local entry is verified, should not get merged.
-  AutofillProfile local = AutofillProfile(kGuidA, kSettingsOrigin);
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kSettingsOrigin);
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile is similar to the local one.
-  AutofillProfile remote = AutofillProfile(kGuidB, kHttpOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-  remote.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
-
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kHttpOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
   MergeSimilarEntriesForInitialSync();
 
   // The local profile gets uploaded (due to initial sync) and the remote
   // profile gets stored locally.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), ElementsAre(local));
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(local));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(local, remote));
 }
 
 TEST_F(AutofillProfileInitialSyncDifferenceTrackerTest,
        MergeSimilarEntriesForInitialSyncDoesNotMatchRemoteVerifiedEntry) {
-  AutofillProfile local = AutofillProfile(kGuidA, kHttpOrigin);
-  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
+  AutofillProfile local = AutofillProfile(kSmallerGuid, kHttpOrigin);
+  local.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  local.FinalizeAfterImport();
   AddAutofillProfilesToTable({local});
 
   // The remote profile is similar to the local one but is verified and thus it
   // should not get merged.
-  AutofillProfile remote = AutofillProfile(kGuidB, kSettingsOrigin);
-  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, ASCIIToUTF16("1 1st st"));
-  remote.SetRawInfo(COMPANY_NAME, ASCIIToUTF16("Frobbers, Inc."));
-
+  AutofillProfile remote = AutofillProfile(kBiggerGuid, kSettingsOrigin);
+  remote.SetRawInfo(ADDRESS_HOME_STREET_ADDRESS, u"1 1st st");
+  remote.SetRawInfo(COMPANY_NAME, u"Frobbers, Inc.");
+  remote.FinalizeAfterImport();
   IncorporateRemoteProfile(remote);
   MergeSimilarEntriesForInitialSync();
 
   // The local profile gets uploaded (due to initial sync) and the remote
   // profile gets stored locally.
-  EXPECT_THAT(FlushAndReturnProfilesToUploadToSync(), ElementsAre(local));
+  UpdatesToSync updates = FlushToSync();
+  EXPECT_THAT(updates.profiles_to_upload_to_sync, ElementsAre(local));
+  EXPECT_THAT(updates.profiles_to_delete_from_sync, IsEmpty());
   EXPECT_THAT(GetAllLocalData(), ElementsAre(local, remote));
 }
 
