@@ -9,7 +9,7 @@
 
 #include <string>
 
-#include "base/macros.h"
+#include "components/autofill/core/common/form_data.h"
 #include "url/gurl.h"
 
 namespace base {
@@ -18,8 +18,6 @@ class Value;
 
 namespace autofill {
 
-struct PasswordForm;
-
 // When logging decisions made by password management code about whether to
 // offer user-entered credentials for saving or not, do use this class. It
 // offers a suite of convenience methods to format and scrub logs. The methods
@@ -69,30 +67,18 @@ class SavePasswordProgressLogger {
     STRING_FORM_FOUND_ON_PAGE,
     STRING_FORM_IS_VISIBLE,
     STRING_FORM_IS_PASSWORD,
-    STRING_FORM_IS_NOT_PASSWORD,
-    STRING_WILL_SUBMIT_FORM_METHOD,
     STRING_HTML_FORM_FOR_SUBMIT,
-    STRING_CREATED_PASSWORD_FORM,
-    STRING_SUBMITTED_PASSWORD_REPLACED,
     STRING_DID_START_PROVISIONAL_LOAD_METHOD,
     STRING_FRAME_NOT_MAIN_FRAME,
-    STRING_PROVISIONALLY_SAVED_FORM_FOR_FRAME,
     STRING_PROVISIONALLY_SAVE_FORM_METHOD,
-    STRING_PROVISIONALLY_SAVE_PASSWORD_METHOD,
-    STRING_PROVISIONALLY_SAVE_PASSWORD_FORM,
-    STRING_IS_SAVING_ENABLED,
     STRING_EMPTY_PASSWORD,
-    STRING_EXACT_MATCH,
-    STRING_MATCH_WITHOUT_ACTION,
-    STRING_ORIGINS_MATCH,
     STRING_MATCHING_NOT_COMPLETE,
-    STRING_FORM_BLACKLISTED,
     STRING_INVALID_FORM,
     STRING_SYNC_CREDENTIAL,
     STRING_BLOCK_PASSWORD_SAME_ORIGIN_INSECURE_SCHEME,
-    STRING_PROVISIONALLY_SAVED_FORM,
     STRING_ON_PASSWORD_FORMS_RENDERED_METHOD,
-    STRING_ON_SAME_DOCUMENT_NAVIGATION,
+    STRING_ON_DYNAMIC_FORM_SUBMISSION,
+    STRING_ON_SUBFRAME_FORM_SUBMISSION,
     STRING_ON_ASK_USER_OR_SAVE_PASSWORD,
     STRING_CAN_PROVISIONAL_MANAGER_SAVE_METHOD,
     STRING_NO_PROVISIONAL_SAVE_MANAGER,
@@ -104,38 +90,22 @@ class SavePasswordProgressLogger {
     STRING_ONLY_VISIBLE,
     STRING_SHOW_PASSWORD_PROMPT,
     STRING_PASSWORDMANAGER_AUTOFILL,
-    STRING_PASSWORDMANAGER_AUTOFILLHTTPAUTH,
-    STRING_PASSWORDMANAGER_SHOW_INITIAL_PASSWORD_ACCOUNT_SUGGESTIONS,
     STRING_WAIT_FOR_USERNAME,
     STRING_WAS_LAST_NAVIGATION_HTTP_ERROR_METHOD,
     STRING_HTTP_STATUS_CODE,
     STRING_PROVISIONALLY_SAVED_FORM_IS_NOT_HTML,
-    STRING_ON_FETCH_COMPLETED_METHOD,
-    STRING_BEST_SCORE,
     STRING_ON_GET_STORE_RESULTS_METHOD,
     STRING_NUMBER_RESULTS,
     STRING_FETCH_METHOD,
     STRING_NO_STORE,
     STRING_CREATE_LOGIN_MANAGERS_METHOD,
-    STRING_OLD_NUMBER_LOGIN_MANAGERS,
-    STRING_NEW_NUMBER_LOGIN_MANAGERS,
     STRING_PASSWORD_MANAGEMENT_ENABLED_FOR_CURRENT_PAGE,
     STRING_SHOW_LOGIN_PROMPT_METHOD,
     STRING_NEW_UI_STATE,
-    STRING_FORM_NOT_AUTOFILLED,
-    STRING_CHANGE_PASSWORD_FORM,
-    STRING_PROCESS_FRAME_METHOD,
     STRING_FORM_SIGNATURE,
-    STRING_ADDING_SIGNATURE,
     STRING_FORM_FETCHER_STATE,
     STRING_UNOWNED_INPUTS_VISIBLE,
     STRING_ON_FILL_PASSWORD_FORM_METHOD,
-    STRING_ON_SHOW_INITIAL_PASSWORD_ACCOUNT_SUGGESTIONS,
-    STRING_AMBIGUOUS_OR_EMPTY_NAMES,
-    STRING_NUMBER_OF_POTENTIAL_FORMS_TO_FILL,
-    STRING_CONTAINS_FILLABLE_USERNAME_FIELD,
-    STRING_USERNAME_FIELD_NAME_EMPTY,
-    STRING_PASSWORD_FIELD_NAME_EMPTY,
     STRING_FORM_DATA_WAIT,
     STRING_FILL_USERNAME_AND_PASSWORD_METHOD,
     STRING_USERNAMES_MATCH,
@@ -144,24 +114,13 @@ class SavePasswordProgressLogger {
     STRING_PASSWORD_FILLED,
     STRING_FORM_NAME,
     STRING_FIELDS,
-    STRING_SERVER_PREDICTIONS,
-    STRING_FORM_VOTES,
+    STRING_FIRSTUSE_FORM_VOTE,
+    STRING_PASSWORD_FORM_VOTE,
     STRING_REUSE_FOUND,
     STRING_GENERATION_DISABLED_SAVING_DISABLED,
     STRING_GENERATION_DISABLED_NO_SYNC,
-    STRING_GENERATION_RENDERER_ENABLED,
-    STRING_GENERATION_RENDERER_INVALID_PASSWORD_FORM,
-    STRING_GENERATION_RENDERER_POSSIBLE_ACCOUNT_CREATION_FORMS,
-    STRING_GENERATION_RENDERER_NO_PASSWORD_MANAGER_ACCESS,
-    STRING_GENERATION_RENDERER_FORM_ALREADY_FOUND,
-    STRING_GENERATION_RENDERER_NO_POSSIBLE_CREATION_FORMS,
-    STRING_GENERATION_RENDERER_NOT_BLACKLISTED,
-    STRING_GENERATION_RENDERER_AUTOCOMPLETE_ATTRIBUTE,
-    STRING_GENERATION_RENDERER_NO_SERVER_SIGNAL,
-    STRING_GENERATION_RENDERER_ELIGIBLE_FORM_FOUND,
-    STRING_GENERATION_RENDERER_NO_FIELD_FOUND,
     STRING_GENERATION_RENDERER_AUTOMATIC_GENERATION_AVAILABLE,
-    STRING_GENERATION_RENDERER_SHOW_MANUAL_GENERATION_POPUP,
+    STRING_GENERATION_RENDERER_SHOW_GENERATION_POPUP,
     STRING_GENERATION_RENDERER_GENERATED_PASSWORD_ACCEPTED,
     STRING_SUCCESSFUL_SUBMISSION_INDICATOR_EVENT,
     STRING_MAIN_FRAME_ORIGIN,
@@ -172,21 +131,46 @@ class SavePasswordProgressLogger {
     STRING_FAILED_TO_FILL_NO_AUTOCOMPLETEABLE_ELEMENT,
     STRING_FAILED_TO_FILL_PREFILLED_USERNAME,
     STRING_FAILED_TO_FILL_FOUND_NO_PASSWORD_FOR_USERNAME,
-    STRING_HTTPAUTH_OBSERVER_PRESENT,
     STRING_HTTPAUTH_ON_ASK_USER_OR_SAVE_PASSWORD,
     STRING_HTTPAUTH_ON_PROMPT_USER,
     STRING_HTTPAUTH_ON_SET_OBSERVER,
     STRING_HTTPAUTH_ON_DETACH_OBSERVER,
+    STRING_LEAK_DETECTION_DISABLED_FEATURE,
+    STRING_LEAK_DETECTION_DISABLED_SAFE_BROWSING,
+    STRING_LEAK_DETECTION_FINISHED,
+    STRING_LEAK_DETECTION_HASH_ERROR,
+    STRING_LEAK_DETECTION_INVALID_SERVER_RESPONSE_ERROR,
+    STRING_LEAK_DETECTION_SIGNED_OUT_ERROR,
+    STRING_LEAK_DETECTION_TOKEN_REQUEST_ERROR,
+    STRING_LEAK_DETECTION_NETWORK_ERROR,
+    STRING_LEAK_DETECTION_QUOTA_LIMIT,
+    STRING_PASSWORD_REQUIREMENTS_VOTE_FOR_LOWERCASE,
+    STRING_PASSWORD_REQUIREMENTS_VOTE_FOR_SPECIAL_SYMBOL,
+    STRING_PASSWORD_REQUIREMENTS_VOTE_FOR_SPECIFIC_SPECIAL_SYMBOL,
+    STRING_PASSWORD_REQUIREMENTS_VOTE_FOR_PASSWORD_LENGTH,
+    STRING_SAVE_PASSWORD_HASH,
+    STRING_DID_NAVIGATE_MAIN_FRAME,
+    STRING_NAVIGATION_NTP,
+    STRING_SERVER_PREDICTIONS,
+    STRING_USERNAME_FIRST_FLOW_VOTE,
+    STRING_POSSIBLE_USERNAME_USED,
+    STRING_POSSIBLE_USERNAME_NOT_USED,
+    STRING_LOCALLY_SAVED_PREDICTION,
     STRING_INVALID,  // Represents a string returned in a case of an error.
     STRING_MAX = STRING_INVALID
   };
 
   SavePasswordProgressLogger();
+
+  SavePasswordProgressLogger(const SavePasswordProgressLogger&) = delete;
+  SavePasswordProgressLogger& operator=(const SavePasswordProgressLogger&) =
+      delete;
+
   virtual ~SavePasswordProgressLogger();
 
   // Call these methods to log information. They sanitize the input and call
   // SendLog to pass it for display.
-  void LogPasswordForm(StringID label, const PasswordForm& form);
+  void LogFormData(StringID label, const FormData& form_data);
   void LogHTMLForm(StringID label,
                    const std::string& name_or_id,
                    const GURL& action);
@@ -212,16 +196,13 @@ class SavePasswordProgressLogger {
   // will be still possible to match the scrubbed string to the original ID or
   // name in the HTML doc. That's good enough for the logging purposes, and
   // provides some security benefits.
-  static std::string ScrubElementID(const base::string16& element_id);
+  static std::string ScrubElementID(const std::u16string& element_id);
 
   // The UTF-8 version of the function above.
   static std::string ScrubElementID(std::string element_id);
 
   // Translates the StringID values into the corresponding strings.
   static std::string GetStringFromID(SavePasswordProgressLogger::StringID id);
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(SavePasswordProgressLogger);
 };
 
 }  // namespace autofill
