@@ -16,6 +16,7 @@
 #include "base/metrics/histogram_macros.h"
 #include "base/time/default_clock.h"
 #include "base/trace_event/trace_event.h"
+#include "build/build_config.h"
 #include "components/content_settings/core/browser/content_settings_info.h"
 #include "components/content_settings/core/browser/content_settings_pref.h"
 #include "components/content_settings/core/browser/content_settings_registry.h"
@@ -25,6 +26,7 @@
 #include "components/content_settings/core/browser/website_settings_registry.h"
 #include "components/content_settings/core/common/content_settings.h"
 #include "components/content_settings/core/common/content_settings_pattern.h"
+#include "components/content_settings/core/common/content_settings_utils.h"
 #include "components/content_settings/core/common/pref_names.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/pref_registry.h"
@@ -32,6 +34,8 @@
 #include "components/prefs/scoped_user_pref_update.h"
 #include "services/preferences/public/cpp/dictionary_value_update.h"
 #include "services/preferences/public/cpp/scoped_pref_update.h"
+#include "services/tracing/public/cpp/perfetto/macros.h"
+#include "third_party/perfetto/protos/perfetto/trace/track_event/chrome_content_settings_event_info.pbzero.h"
 
 namespace content_settings {
 
@@ -41,14 +45,26 @@ namespace {
 const char kObsoleteDomainToOriginMigrationStatus[] =
     "profile.content_settings.domain_to_origin_migration_status";
 
-#if !defined(OS_IOS)
+#if !BUILDFLAG(IS_IOS)
 const char kObsoleteFullscreenExceptionsPref[] =
     "profile.content_settings.exceptions.fullscreen";
-#if !defined(OS_ANDROID)
+// The "nfc" preference was superseded by "nfc-devices" once Web NFC gained the
+// ability to make NFC tags permanently read-only. See crbug.com/1275576
+const char kObsoleteNfcExceptionsPref[] =
+    "profile.content_settings.exceptions.nfc";
+#if !BUILDFLAG(IS_ANDROID)
 const char kObsoleteMouseLockExceptionsPref[] =
     "profile.content_settings.exceptions.mouselock";
-#endif  // !defined(OS_ANDROID)
-#endif  // !defined(OS_IOS)
+const char kObsoletePluginsExceptionsPref[] =
+    "profile.content_settings.exceptions.plugins";
+const char kObsoletePluginsDataExceptionsPref[] =
+    "profile.content_settings.exceptions.flash_data";
+const char kObsoleteFileHandlingExceptionsPref[] =
+    "profile.content_settings.exceptions.file_handling";
+const char kObsoleteFontAccessExceptionsPref[] =
+    "profile.content_settings.exceptions.font_access";
+#endif  // !BUILDFLAG(IS_ANDROID)
+#endif  // !BUILDFLAG(IS_IOS)
 
 }  // namespace
 
@@ -75,26 +91,32 @@ void PrefProvider::RegisterProfilePrefs(
   // These prefs have been removed, but need to be registered so they can
   // be deleted on startup.
   registry->RegisterIntegerPref(kObsoleteDomainToOriginMigrationStatus, 0);
-#if !defined(OS_IOS)
+#if !BUILDFLAG(IS_IOS)
   registry->RegisterDictionaryPref(
       kObsoleteFullscreenExceptionsPref,
       user_prefs::PrefRegistrySyncable::SYNCABLE_PREF);
-#if !defined(OS_ANDROID)
+  registry->RegisterDictionaryPref(kObsoleteNfcExceptionsPref);
+#if !BUILDFLAG(IS_ANDROID)
   registry->RegisterDictionaryPref(
       kObsoleteMouseLockExceptionsPref,
       user_prefs::PrefRegistrySyncable::SYNCABLE_PREF);
-#endif  // !defined(OS_ANDROID)
-#endif  // !defined(OS_IOS)
+  registry->RegisterDictionaryPref(kObsoletePluginsDataExceptionsPref);
+  registry->RegisterDictionaryPref(kObsoletePluginsExceptionsPref);
+  registry->RegisterDictionaryPref(kObsoleteFileHandlingExceptionsPref);
+  registry->RegisterDictionaryPref(kObsoleteFontAccessExceptionsPref);
+#endif  // !BUILDFLAG(IS_ANDROID)
+#endif  // !BUILDFLAG(IS_IOS)
 }
 
 PrefProvider::PrefProvider(PrefService* prefs,
-                           bool incognito,
-                           bool store_last_modified)
+                           bool off_the_record,
+                           bool store_last_modified,
+                           bool restore_session)
     : prefs_(prefs),
-      is_incognito_(incognito),
+      off_the_record_(off_the_record),
       store_last_modified_(store_last_modified),
       clock_(base::DefaultClock::GetInstance()) {
-  TRACE_EVENT_BEGIN0("startup", "PrefProvider::PrefProvider");
+  TRACE_EVENT_BEGIN("startup", "PrefProvider::PrefProvider");
   DCHECK(prefs_);
   // Verify preferences version.
   if (!prefs_->HasPrefPath(prefs::kContentSettingsVersion)) {
@@ -103,11 +125,11 @@ PrefProvider::PrefProvider(PrefService* prefs,
   }
   if (prefs_->GetInteger(prefs::kContentSettingsVersion) >
       ContentSettingsPattern::kContentSettingsPatternVersion) {
-    TRACE_EVENT_END0("startup", "PrefProvider::PrefProvider");
+    TRACE_EVENT_END("startup");  // PrefProvider::PrefProvider.
     return;
   }
 
-  DiscardObsoletePreferences();
+  DiscardOrMigrateObsoletePreferences();
 
   pref_change_registrar_.Init(prefs_);
 
@@ -123,23 +145,16 @@ PrefProvider::PrefProvider(PrefService* prefs,
     if (!content_type_info || content_type_info->storage_behavior() ==
                                   ContentSettingsInfo::PERSISTENT) {
       content_settings_prefs_.insert(std::make_pair(
-          info->type(),
-          std::make_unique<ContentSettingsPref>(
-              info->type(), prefs_, &pref_change_registrar_, info->pref_name(),
-              is_incognito_,
-              base::Bind(&PrefProvider::Notify, base::Unretained(this)))));
-    } else if (info->type() == CONTENT_SETTINGS_TYPE_PLUGINS) {
-      // TODO(https://crbug.com/850062): Remove after M71, two milestones after
-      // migration of the Flash permissions to ephemeral provider.
-      flash_content_settings_pref_ = std::make_unique<ContentSettingsPref>(
-          info->type(), prefs_, &pref_change_registrar_, info->pref_name(),
-          is_incognito_,
-          base::Bind(&PrefProvider::Notify, base::Unretained(this)));
+          info->type(), std::make_unique<ContentSettingsPref>(
+                            info->type(), prefs_, &pref_change_registrar_,
+                            info->pref_name(), off_the_record_, restore_session,
+                            base::BindRepeating(&PrefProvider::Notify,
+                                                base::Unretained(this)))));
     }
   }
 
   size_t num_exceptions = 0;
-  if (!is_incognito_) {
+  if (!off_the_record_) {
     for (const auto& pref : content_settings_prefs_)
       num_exceptions += pref.second->GetNumExceptions();
 
@@ -147,8 +162,12 @@ PrefProvider::PrefProvider(PrefService* prefs,
                             num_exceptions);
   }
 
-  TRACE_EVENT_END1("startup", "PrefProvider::PrefProvider",
-                   "NumberOfExceptions", num_exceptions);
+  TRACE_EVENT_END("startup", [num_exceptions](perfetto::EventContext ctx) {
+    perfetto::protos::pbzero::ChromeContentSettingsEventInfo* event_args =
+        ctx.event()->set_chrome_content_settings_event_info();
+    event_args->set_number_of_exceptions(
+        num_exceptions);  // PrefProvider::PrefProvider.
+  });
 }
 
 PrefProvider::~PrefProvider() {
@@ -157,20 +176,19 @@ PrefProvider::~PrefProvider() {
 
 std::unique_ptr<RuleIterator> PrefProvider::GetRuleIterator(
     ContentSettingsType content_type,
-    const ResourceIdentifier& resource_identifier,
-    bool incognito) const {
+    bool off_the_record) const {
   if (!supports_type(content_type))
     return nullptr;
 
-  return GetPref(content_type)->GetRuleIterator(resource_identifier, incognito);
+  return GetPref(content_type)->GetRuleIterator(off_the_record);
 }
 
 bool PrefProvider::SetWebsiteSetting(
     const ContentSettingsPattern& primary_pattern,
     const ContentSettingsPattern& secondary_pattern,
     ContentSettingsType content_type,
-    const ResourceIdentifier& resource_identifier,
-    base::Value* in_value) {
+    base::Value&& in_value,
+    const ContentSettingConstraints& constraints) {
   DCHECK(CalledOnValidThread());
   DCHECK(prefs_);
 
@@ -183,24 +201,31 @@ bool PrefProvider::SetWebsiteSetting(
   // sites/origins defined by the |primary_pattern| and the |secondary_pattern|.
   // Default settings are handled by the |DefaultProvider|.
   if (primary_pattern == ContentSettingsPattern::Wildcard() &&
-      secondary_pattern == ContentSettingsPattern::Wildcard() &&
-      resource_identifier.empty()) {
+      secondary_pattern == ContentSettingsPattern::Wildcard()) {
     return false;
   }
 
   base::Time modified_time =
       store_last_modified_ ? clock_->Now() : base::Time();
 
-  return GetPref(content_type)
-      ->SetWebsiteSetting(primary_pattern, secondary_pattern,
-                          resource_identifier, modified_time, in_value);
+  // If SessionModel is OneTime, we know for sure that a one time permission
+  // has been set by the One Time Provider, therefore we reset a potentially
+  // existing Allow Always setting.
+  if (constraints.session_model == SessionModel::OneTime) {
+    DCHECK_EQ(content_type, ContentSettingsType::GEOLOCATION);
+    in_value = base::Value();
+  }
+
+  GetPref(content_type)
+      ->SetWebsiteSetting(primary_pattern, secondary_pattern, modified_time,
+                          std::move(in_value), constraints);
+  return true;
 }
 
 base::Time PrefProvider::GetWebsiteSettingLastModified(
     const ContentSettingsPattern& primary_pattern,
     const ContentSettingsPattern& secondary_pattern,
-    ContentSettingsType content_type,
-    const ResourceIdentifier& resource_identifier) {
+    ContentSettingsType content_type) {
   DCHECK(CalledOnValidThread());
   DCHECK(prefs_);
 
@@ -208,8 +233,7 @@ base::Time PrefProvider::GetWebsiteSettingLastModified(
     return base::Time();
 
   return GetPref(content_type)
-      ->GetWebsiteSettingLastModified(primary_pattern, secondary_pattern,
-                                      resource_identifier);
+      ->GetWebsiteSettingLastModified(primary_pattern, secondary_pattern);
 }
 
 void PrefProvider::ClearAllContentSettingsRules(
@@ -220,14 +244,6 @@ void PrefProvider::ClearAllContentSettingsRules(
   if (supports_type(content_type))
     GetPref(content_type)->ClearAllContentSettingsRules();
 
-  // TODO(https://crbug.com/850062): Remove after M71, two milestones after
-  // migration of the Flash permissions to ephemeral provider.
-  // |flash_content_settings_pref_| is not null only if Flash permissions are
-  // ephemeral and handled in EphemeralProvider.
-  if (content_type == CONTENT_SETTINGS_TYPE_PLUGINS &&
-      flash_content_settings_pref_) {
-    flash_content_settings_pref_->ClearAllContentSettingsRules();
-  }
 }
 
 void PrefProvider::ShutdownOnUIThread() {
@@ -252,31 +268,31 @@ ContentSettingsPref* PrefProvider::GetPref(ContentSettingsType type) const {
   return it->second.get();
 }
 
-void PrefProvider::Notify(
-    const ContentSettingsPattern& primary_pattern,
-    const ContentSettingsPattern& secondary_pattern,
-    ContentSettingsType content_type,
-    const std::string& resource_identifier) {
-  NotifyObservers(primary_pattern,
-                  secondary_pattern,
-                  content_type,
-                  resource_identifier);
+void PrefProvider::Notify(const ContentSettingsPattern& primary_pattern,
+                          const ContentSettingsPattern& secondary_pattern,
+                          ContentSettingsType content_type) {
+  NotifyObservers(primary_pattern, secondary_pattern, content_type);
 }
 
-void PrefProvider::DiscardObsoletePreferences() {
-  if (is_incognito_)
+void PrefProvider::DiscardOrMigrateObsoletePreferences() {
+  if (off_the_record_)
     return;
 
   prefs_->ClearPref(kObsoleteDomainToOriginMigrationStatus);
 
   // These prefs were never stored on iOS/Android so they don't need to be
   // deleted.
-#if !defined(OS_IOS)
+#if !BUILDFLAG(IS_IOS)
   prefs_->ClearPref(kObsoleteFullscreenExceptionsPref);
-#if !defined(OS_ANDROID)
+  prefs_->ClearPref(kObsoleteNfcExceptionsPref);
+#if !BUILDFLAG(IS_ANDROID)
   prefs_->ClearPref(kObsoleteMouseLockExceptionsPref);
-#endif  // !defined(OS_ANDROID)
-#endif  // !defined(OS_IOS)
+  prefs_->ClearPref(kObsoletePluginsExceptionsPref);
+  prefs_->ClearPref(kObsoletePluginsDataExceptionsPref);
+  prefs_->ClearPref(kObsoleteFileHandlingExceptionsPref);
+  prefs_->ClearPref(kObsoleteFontAccessExceptionsPref);
+#endif  // !BUILDFLAG(IS_ANDROID)
+#endif  // !BUILDFLAG(IS_IOS)
 }
 
 void PrefProvider::SetClockForTesting(base::Clock* clock) {
