@@ -10,16 +10,18 @@
 #include <vector>
 
 #include "base/callback.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/memory/weak_ptr.h"
-#include "base/single_thread_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
+#include "base/time/time.h"
 #include "components/download/internal/background_service/blob_task_proxy.h"
 #include "components/download/public/background_service/blob_context_getter_factory.h"
 #include "components/download/public/background_service/download_params.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 #include "services/network/public/cpp/simple_url_loader_stream_consumer.h"
 #include "services/network/public/mojom/url_loader_factory.mojom.h"
+#include "services/network/public/mojom/url_response_head.mojom-forward.h"
 
 class GURL;
 
@@ -96,6 +98,9 @@ class InMemoryDownload {
     COMPLETE,
   };
 
+  InMemoryDownload(const InMemoryDownload&) = delete;
+  InMemoryDownload& operator=(const InMemoryDownload&) = delete;
+
   virtual ~InMemoryDownload();
 
   // Send the download request.
@@ -147,9 +152,6 @@ class InMemoryDownload {
   uint64_t bytes_downloaded_;
 
   uint64_t bytes_uploaded_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(InMemoryDownload);
 };
 
 // Implementation of InMemoryDownload and uses SimpleURLLoader as network
@@ -172,6 +174,9 @@ class InMemoryDownloadImpl : public network::SimpleURLLoaderStreamConsumer,
       network::mojom::URLLoaderFactory* url_loader_factory,
       scoped_refptr<base::SingleThreadTaskRunner> io_task_runner);
 
+  InMemoryDownloadImpl(const InMemoryDownloadImpl&) = delete;
+  InMemoryDownloadImpl& operator=(const InMemoryDownloadImpl&) = delete;
+
   ~InMemoryDownloadImpl() override;
 
  private:
@@ -207,12 +212,12 @@ class InMemoryDownloadImpl : public network::SimpleURLLoaderStreamConsumer,
 
   // Called when the server redirects to another URL.
   void OnRedirect(const net::RedirectInfo& redirect_info,
-                  const network::ResourceResponseHead& response_head,
+                  const network::mojom::URLResponseHead& response_head,
                   std::vector<std::string>* to_be_removed_headers);
 
   // Called when the response of the final URL is received.
   void OnResponseStarted(const GURL& final_url,
-                         const network::ResourceResponseHead& response_head);
+                         const network::mojom::URLResponseHead& response_head);
 
   void OnUploadProgress(uint64_t position, uint64_t total);
 
@@ -232,7 +237,7 @@ class InMemoryDownloadImpl : public network::SimpleURLLoaderStreamConsumer,
   std::unique_ptr<network::SimpleURLLoader> loader_;
 
   // Used to handle network response.
-  network::mojom::URLLoaderFactory* url_loader_factory_;
+  raw_ptr<network::mojom::URLLoaderFactory> url_loader_factory_;
 
   // Worker that does blob related task on IO thread.
   std::unique_ptr<BlobTaskProxy> blob_task_proxy_;
@@ -244,7 +249,7 @@ class InMemoryDownloadImpl : public network::SimpleURLLoaderStreamConsumer,
   // Used to access blob storage context.
   scoped_refptr<base::SingleThreadTaskRunner> io_task_runner_;
 
-  Delegate* delegate_;
+  raw_ptr<Delegate> delegate_;
 
   // Data downloaded from network, should be moved to avoid extra copy.
   std::string data_;
@@ -260,8 +265,6 @@ class InMemoryDownloadImpl : public network::SimpleURLLoaderStreamConsumer,
 
   // Bounded to main thread task runner.
   base::WeakPtrFactory<InMemoryDownloadImpl> weak_ptr_factory_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(InMemoryDownloadImpl);
 };
 
 }  // namespace download
