@@ -6,7 +6,6 @@
 
 #include "base/bind.h"
 #include "base/containers/contains.h"
-#include "base/task/post_task.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "components/download/database/download_db_entry.h"
@@ -30,7 +29,7 @@
 #include "services/network/public/mojom/url_response_head.mojom.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 #include "base/android/build_info.h"
 #include "components/download/internal/common/android/download_collection_bridge.h"
 #include "components/download/public/common/download_path_reservation_tracker.h"
@@ -55,22 +54,24 @@ std::unique_ptr<DownloadItemImpl> CreateDownloadItemImpl(
       entry.download_info->in_progress_info;
   if (!in_progress_info)
     return nullptr;
+
   return std::make_unique<DownloadItemImpl>(
       delegate, entry.download_info->guid, entry.download_info->id,
       in_progress_info->current_path, in_progress_info->target_path,
       in_progress_info->url_chain, in_progress_info->referrer_url,
-      in_progress_info->site_url, in_progress_info->tab_url,
-      in_progress_info->tab_referrer_url, absl::nullopt,
-      in_progress_info->mime_type, in_progress_info->original_mime_type,
-      in_progress_info->start_time, in_progress_info->end_time,
-      in_progress_info->etag, in_progress_info->last_modified,
-      in_progress_info->received_bytes, in_progress_info->total_bytes,
-      in_progress_info->auto_resume_count, in_progress_info->hash,
-      in_progress_info->state, in_progress_info->danger_type,
-      in_progress_info->interrupt_reason, in_progress_info->paused,
-      in_progress_info->metered, false, base::Time(),
+      in_progress_info->serialized_embedder_download_data,
+      in_progress_info->tab_url, in_progress_info->tab_referrer_url,
+      absl::nullopt, in_progress_info->mime_type,
+      in_progress_info->original_mime_type, in_progress_info->start_time,
+      in_progress_info->end_time, in_progress_info->etag,
+      in_progress_info->last_modified, in_progress_info->received_bytes,
+      in_progress_info->total_bytes, in_progress_info->auto_resume_count,
+      in_progress_info->hash, in_progress_info->state,
+      in_progress_info->danger_type, in_progress_info->interrupt_reason,
+      in_progress_info->paused, in_progress_info->metered, false, base::Time(),
       in_progress_info->transient, in_progress_info->received_slices,
       in_progress_info->reroute_info, in_progress_info->download_schedule,
+      in_progress_info->range_request_from, in_progress_info->range_request_to,
       std::move(download_entry));
 }
 
@@ -92,7 +93,7 @@ void BeginResourceDownload(
     const URLSecurityPolicy& url_security_policy,
     bool is_new_download,
     base::WeakPtr<InProgressDownloadManager> download_manager,
-    const GURL& site_url,
+    const std::string& serialized_embedder_download_data,
     const GURL& tab_url,
     const GURL& tab_referrer_url,
     mojo::PendingRemote<device::mojom::WakeLockProvider> wake_lock_provider,
@@ -104,9 +105,9 @@ void BeginResourceDownload(
           download_manager, std::move(params), std::move(request),
           network::SharedURLLoaderFactory::Create(
               std::move(pending_url_loader_factory)),
-          url_security_policy, site_url, tab_url, tab_referrer_url,
-          is_new_download, false, std::move(wake_lock_provider),
-          is_background_mode, main_task_runner)
+          url_security_policy, serialized_embedder_download_data, tab_url,
+          tab_referrer_url, is_new_download, false,
+          std::move(wake_lock_provider), is_background_mode, main_task_runner)
           .release(),
       base::OnTaskRunnerDeleter(base::ThreadTaskRunnerHandle::Get()));
 
@@ -119,7 +120,7 @@ void CreateDownloadHandlerForNavigation(
     std::unique_ptr<network::ResourceRequest> resource_request,
     int render_process_id,
     int render_frame_id,
-    const GURL& site_url,
+    const std::string& serialized_embedder_download_data,
     const GURL& tab_url,
     const GURL& tab_referrer_url,
     std::vector<GURL> url_chain,
@@ -136,15 +137,16 @@ void CreateDownloadHandlerForNavigation(
 
   ResourceDownloader::InterceptNavigationResponse(
       download_manager, std::move(resource_request), render_process_id,
-      render_frame_id, site_url, tab_url, tab_referrer_url,
-      std::move(url_chain), std::move(cert_status), std::move(response_head),
-      std::move(response_body), std::move(url_loader_client_endpoints),
+      render_frame_id, serialized_embedder_download_data, tab_url,
+      tab_referrer_url, std::move(url_chain), std::move(cert_status),
+      std::move(response_head), std::move(response_body),
+      std::move(url_loader_client_endpoints),
       network::SharedURLLoaderFactory::Create(
           std::move(pending_url_loader_factory)),
       url_security_policy, std::move(wake_lock_provider), main_task_runner);
 }
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 void OnDownloadDisplayNamesReturned(
     DownloadCollectionBridge::GetDisplayNamesCallback callback,
     const scoped_refptr<base::SingleThreadTaskRunner>& main_task_runner,
@@ -181,7 +183,8 @@ void OnPathReserved(DownloadItemImplDelegate::DownloadTargetCallback callback,
           : BackgroudTargetDeterminationResultTypes::kSuccess);
   std::move(callback).Run(
       target_path, DownloadItem::TARGET_DISPOSITION_OVERWRITE, danger_type,
-      mixed_content_status, intermediate_path, std::move(download_schedule),
+      mixed_content_status, intermediate_path, base::FilePath(),
+      std::move(download_schedule),
       intermediate_path.empty() ? DOWNLOAD_INTERRUPT_REASON_FILE_FAILED
                                 : DOWNLOAD_INTERRUPT_REASON_NONE);
 }
@@ -268,7 +271,8 @@ void InProgressDownloadManager::DownloadUrl(
   // Start the new download, the download should be saved to the file path
   // specifcied in the |params|.
   BeginDownload(std::move(params), url_loader_factory_->Clone(),
-                true /* is_new_download */, GURL() /* site_url */,
+                true /* is_new_download */,
+                std::string() /* serialized_embedder_download_data */,
                 GURL() /* tab_url */, GURL() /* tab_referral_url */);
 }
 
@@ -308,7 +312,7 @@ void InProgressDownloadManager::BeginDownload(
     std::unique_ptr<network::PendingSharedURLLoaderFactory>
         pending_url_loader_factory,
     bool is_new_download,
-    const GURL& site_url,
+    const std::string& serialized_embedder_download_data,
     const GURL& tab_url,
     const GURL& tab_referrer_url) {
   std::unique_ptr<network::ResourceRequest> request =
@@ -320,20 +324,20 @@ void InProgressDownloadManager::BeginDownload(
   }
   GetIOTaskRunner()->PostTask(
       FROM_HERE,
-      base::BindOnce(&BeginResourceDownload, std::move(params),
-                     std::move(request), std::move(pending_url_loader_factory),
-                     url_security_policy_, is_new_download,
-                     weak_factory_.GetWeakPtr(), site_url, tab_url,
-                     tab_referrer_url, std::move(wake_lock_provider),
-                     !delegate_ /* is_background_mode */,
-                     base::ThreadTaskRunnerHandle::Get()));
+      base::BindOnce(
+          &BeginResourceDownload, std::move(params), std::move(request),
+          std::move(pending_url_loader_factory), url_security_policy_,
+          is_new_download, weak_factory_.GetWeakPtr(),
+          serialized_embedder_download_data, tab_url, tab_referrer_url,
+          std::move(wake_lock_provider), !delegate_ /* is_background_mode */,
+          base::ThreadTaskRunnerHandle::Get()));
 }
 
 void InProgressDownloadManager::InterceptDownloadFromNavigation(
     std::unique_ptr<network::ResourceRequest> resource_request,
     int render_process_id,
     int render_frame_id,
-    const GURL& site_url,
+    const std::string& serialized_embedder_download_data,
     const GURL& tab_url,
     const GURL& tab_referrer_url,
     std::vector<GURL> url_chain,
@@ -354,9 +358,10 @@ void InProgressDownloadManager::InterceptDownloadFromNavigation(
       base::BindOnce(
           &CreateDownloadHandlerForNavigation, weak_factory_.GetWeakPtr(),
           std::move(resource_request), render_process_id, render_frame_id,
-          site_url, tab_url, tab_referrer_url, std::move(url_chain),
-          std::move(cert_status), std::move(response_head),
-          std::move(response_body), std::move(url_loader_client_endpoints),
+          serialized_embedder_download_data, tab_url, tab_referrer_url,
+          std::move(url_chain), std::move(cert_status),
+          std::move(response_head), std::move(response_body),
+          std::move(url_loader_client_endpoints),
           std::move(pending_url_loader_factory), url_security_policy_,
           std::move(wake_lock_provider), base::ThreadTaskRunnerHandle::Get()));
 }
@@ -394,12 +399,12 @@ void InProgressDownloadManager::DetermineDownloadTarget(
   base::FilePath target_path = download->GetForcedFilePath().empty()
                                    ? download->GetTargetFilePath()
                                    : download->GetForcedFilePath();
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   if (target_path.empty()) {
     std::move(callback).Run(
         target_path, DownloadItem::TARGET_DISPOSITION_OVERWRITE,
         download->GetDangerType(), download->GetMixedContentStatus(),
-        target_path, download->GetDownloadSchedule(),
+        target_path, base::FilePath(), download->GetDownloadSchedule(),
         DOWNLOAD_INTERRUPT_REASON_FILE_FAILED);
     RecordBackgroundTargetDeterminationResult(
         BackgroudTargetDeterminationResultTypes::kTargetPathMissing);
@@ -412,7 +417,7 @@ void InProgressDownloadManager::DetermineDownloadTarget(
     std::move(callback).Run(
         target_path, DownloadItem::TARGET_DISPOSITION_OVERWRITE,
         download->GetDangerType(), download->GetMixedContentStatus(),
-        target_path, download->GetDownloadSchedule(),
+        target_path, base::FilePath(), download->GetDownloadSchedule(),
         DOWNLOAD_INTERRUPT_REASON_NONE);
     RecordBackgroundTargetDeterminationResult(
         BackgroudTargetDeterminationResultTypes::kSuccess);
@@ -436,19 +441,19 @@ void InProgressDownloadManager::DetermineDownloadTarget(
   std::move(callback).Run(
       target_path, DownloadItem::TARGET_DISPOSITION_OVERWRITE,
       download->GetDangerType(), download->GetMixedContentStatus(),
-      intermediate_path, download->GetDownloadSchedule(),
+      intermediate_path, base::FilePath(), download->GetDownloadSchedule(),
       DOWNLOAD_INTERRUPT_REASON_NONE);
-#endif  // defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_ANDROID)
 }
 
 void InProgressDownloadManager::ResumeInterruptedDownload(
     std::unique_ptr<DownloadUrlParameters> params,
-    const GURL& site_url) {
+    const std::string& serialized_embedder_download_data) {
   if (!url_loader_factory_)
     return;
 
   BeginDownload(std::move(params), url_loader_factory_->Clone(), false,
-                site_url, GURL(), GURL());
+                serialized_embedder_download_data, GURL(), GURL());
 }
 
 bool InProgressDownloadManager::ShouldOpenDownload(
@@ -580,7 +585,7 @@ void InProgressDownloadManager::StartDownloadWithItem(
 void InProgressDownloadManager::OnDBInitialized(
     bool success,
     std::unique_ptr<std::vector<DownloadDBEntry>> entries) {
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   // Retrieve display names for all downloads from media store if needed.
   if (base::android::BuildInfo::GetInstance()->sdk_int() >=
       base::android::SDK_VERSION_Q) {
@@ -618,7 +623,7 @@ void InProgressDownloadManager::OnDownloadNamesRetrieved(
       num_duplicates++;
       continue;
     }
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
     const base::FilePath& path = item->GetTargetFilePath();
     if (path.IsContentUri()) {
       base::FilePath display_name = GetDownloadDisplayName(path);
@@ -649,7 +654,7 @@ InProgressDownloadManager::TakeInProgressDownloads() {
 
 base::FilePath InProgressDownloadManager::GetDownloadDisplayName(
     const base::FilePath& path) {
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   if (!display_names_)
     return base::FilePath();
   auto iter = display_names_->find(path.value());
