@@ -24,14 +24,17 @@
 #include <vector>
 
 #include "base/callback_forward.h"
-#include "base/files/file_path.h"
 #include "base/memory/ref_counted.h"
-#include "base/optional.h"
-#include "base/strings/string16.h"
 #include "base/supports_user_data.h"
 #include "components/download/public/common/download_danger_type.h"
 #include "components/download/public/common/download_export.h"
 #include "components/download/public/common/download_interrupt_reasons.h"
+#include "components/download/public/common/download_item_rename_progress_update.h"
+#include "components/download/public/common/download_schedule.h"
+#include "components/download/public/common/download_source.h"
+#include "net/base/isolation_info.h"
+#include "services/network/public/mojom/fetch_api.mojom-shared.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "ui/base/page_transition_types.h"
 #include "url/origin.h"
 
@@ -49,6 +52,7 @@ class HttpResponseHeaders;
 
 namespace download {
 class DownloadFile;
+class DownloadItemRenameHandler;
 
 // One DownloadItem per download. This is the model class that stores all the
 // state for a download.
@@ -99,8 +103,24 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
     RESULT_MAX = FAILURE_UNKNOWN
   };
 
+  // The mixed content status for a download item.
+  enum MixedContentStatus {
+    // Target not yet determined, so status not yet available.
+    UNKNOWN = 0,
+    // Download is not mixed content.
+    SAFE = 1,
+    // Download has been explicitly OK'd by the user. Only used on Desktop.
+    VALIDATED = 2,
+    // Download is mixed content, and the user should be warned.
+    WARN = 3,
+    // Download is mixed content, and the user should see an error.
+    BLOCK = 4,
+    // Download is mixed content, and it should be silently dropped.
+    SILENT_BLOCK = 5,
+  };
+
   // Callback used with AcquireFileAndDeleteDownload().
-  typedef base::Callback<void(const base::FilePath&)> AcquireFileCallback;
+  using AcquireFileCallback = base::OnceCallback<void(const base::FilePath&)>;
   using RenameDownloadCallback = base::OnceCallback<void(DownloadRenameResult)>;
   // Used to represent an invalid download ID.
   static const uint32_t kInvalidId;
@@ -157,6 +177,12 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // Called when the user has validated the download of a dangerous file.
   virtual void ValidateDangerousDownload() = 0;
 
+  // Called when the user has validated the download of a mixed content file.
+  virtual void ValidateMixedContentDownload() = 0;
+
+  // Called when user accepts Incognito download warning.
+  virtual void AcceptIncognitoWarning() = 0;
+
   // Called to acquire a dangerous download. If |delete_file_afterward| is true,
   // invokes |callback| on the UI thread with the path to the downloaded file,
   // and removes the DownloadItem from views and history if appropriate.
@@ -165,7 +191,7 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // Note: It is important for |callback| to be valid since the downloaded file
   // will not be cleaned up if the callback fails.
   virtual void StealDangerousDownload(bool delete_file_afterward,
-                                      const AcquireFileCallback& callback) = 0;
+                                      AcquireFileCallback callback) = 0;
 
   // Pause a download.  Will have no effect if the download is already
   // paused.
@@ -273,10 +299,10 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // URL of document that is considered the referrer for the original URL.
   virtual const GURL& GetReferrerUrl() const = 0;
 
-  // Site instance URL. Used to locate the correct storage partition during
-  // subsequent browser sessions. This may be different from all of
-  // GetOriginalUrl(), GetURL() and GetReferrerUrl().
-  virtual const GURL& GetSiteUrl() const = 0;
+  // The serialized EmbedderDownloadData string. This is used by the embedder
+  // for placing extra download data, such as the appropriate storage partition
+  // for this download.
+  virtual const std::string& GetSerializedEmbedderDownloadData() const = 0;
 
   // URL of the top level frame at the time the download was initiated.
   virtual const GURL& GetTabUrl() const = 0;
@@ -285,7 +311,7 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   virtual const GURL& GetTabReferrerUrl() const = 0;
 
   // Origin of the original originator of this download, before redirects, etc.
-  virtual const base::Optional<url::Origin>& GetRequestInitiator() const = 0;
+  virtual const absl::optional<url::Origin>& GetRequestInitiator() const = 0;
 
   // For downloads initiated via <a download>, this is the suggested download
   // filename from the download attribute.
@@ -327,6 +353,16 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // Whether this download is a SavePackage download.
   virtual bool IsSavePackageDownload() const = 0;
 
+  // DownloadSource prompting this download.
+  virtual DownloadSource GetDownloadSource() const = 0;
+
+  // The credentials mode of the request.
+  virtual ::network::mojom::CredentialsMode GetCredentialsMode() const = 0;
+
+  // The isolation mode of the request.
+  virtual const absl::optional<net::IsolationInfo>& GetIsolationInfo()
+      const = 0;
+
   //    Destination State accessors --------------------------------------------
 
   // Full path to the downloaded or downloading file. This is the path to the
@@ -385,19 +421,43 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // GetFileExternallyRemoved() was already true. The callback will be passed
   // false if the DownloadItem was not yet complete or if the file could not be
   // deleted for any reason.
-  virtual void DeleteFile(const base::Callback<void(bool)>& callback) = 0;
+  virtual void DeleteFile(base::OnceCallback<void(bool)> callback) = 0;
 
   // True if the file that will be written by the download is dangerous
   // and we will require a call to ValidateDangerousDownload() to complete.
   // False if the download is safe or that function has been called.
   virtual bool IsDangerous() const = 0;
 
+  // True if the file that will be written by the download is mixed content
+  // and we will require a call to ValidateMixedContentDownload() to complete.
+  // False if not mixed content or that function has been called.
+  virtual bool IsMixedContent() const = 0;
+
+  // True if file is downloaded in Incognito and user has not accepted it yet.
+  // False if file is downloaded in regular mode or has accepted the incognito
+  // warning.
+  virtual bool ShouldShowIncognitoWarning() const = 0;
+
   // Why |safety_state_| is not SAFE.
   virtual DownloadDangerType GetDangerType() const = 0;
 
+  // Returns the mixed content status of the download, indicating whether the
+  // download should be blocked or the user warned. This may be UNKNOWN if the
+  // download target hasn't been determined.
+  virtual MixedContentStatus GetMixedContentStatus() const = 0;
+
   // Gets the pointer to the DownloadFile owned by this object.
   virtual DownloadFile* GetDownloadFile() = 0;
 
+  // Gets a handler to perform the rename for a download item.  If no special
+  // rename handling is required, this function returns null and the default
+  // rename handling is performed.  The caller does not own the returned
+  // pointer.
+  virtual DownloadItemRenameHandler* GetRenameHandler() = 0;
+
+  // Gets the metadata needed to recover rename handler state.
+  virtual const DownloadItemRerouteInfo& GetRerouteInfo() const = 0;
+
   //    Progress State accessors -----------------------------------------------
 
   // Simple calculation of the amount of time remaining to completion. Fills
@@ -450,6 +510,9 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // Tests if a file type should be opened automatically.
   virtual bool ShouldOpenFileBasedOnExtension() = 0;
 
+  // Tests if a file type should be opened automatically by policy.
+  virtual bool ShouldOpenFileByPolicyBasedOnExtension() = 0;
+
   // Returns true if the download will be auto-opened when complete.
   virtual bool GetOpenWhenComplete() const = 0;
 
@@ -468,6 +531,10 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // for target file path determination.
   virtual bool IsTransient() const = 0;
 
+  // Returns whether the download requires safety checks. Only downloads
+  // triggered by Chrome itself are excluded from safety checks.
+  virtual bool RequireSafetyChecks() const = 0;
+
   // Returns whether the download item corresponds to a parallel download. This
   // usually means parallel download has been enabled and the download job is
   // parallelizable.
@@ -476,6 +543,10 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   // Gets the DownloadCreationType of this item.
   virtual DownloadCreationType GetDownloadCreationType() const = 0;
 
+  // Gets the download schedule to start the time at particular time.
+  virtual const absl::optional<DownloadSchedule>& GetDownloadSchedule()
+      const = 0;
+
   // External state transitions/setters ----------------------------------------
 
   // TODO(rdsmith): These should all be removed; the download item should
@@ -491,6 +562,13 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadItem : public base::SupportsUserData {
   virtual void OnContentCheckCompleted(DownloadDangerType danger_type,
                                        DownloadInterruptReason reason) = 0;
 
+  // Called when async scanning completes with the given |danger_type|.
+  virtual void OnAsyncScanningCompleted(DownloadDangerType danger_type) = 0;
+
+  // Called when the user changes the download schedule options.
+  virtual void OnDownloadScheduleChanged(
+      absl::optional<DownloadSchedule> schedule) = 0;
+
   // Mark the download to be auto-opened when completed.
   virtual void SetOpenWhenComplete(bool open) = 0;
 
