@@ -8,16 +8,19 @@
 #include <string>
 #include <vector>
 
-#include "base/optional.h"
+#include "base/memory/raw_ptr.h"
 #include "components/download/public/common/download_create_info.h"
 #include "components/download/public/common/download_export.h"
 #include "components/download/public/common/download_source.h"
 #include "components/download/public/common/download_stream.mojom.h"
 #include "components/download/public/common/download_url_parameters.h"
 #include "components/download/public/common/download_utils.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "net/cert/cert_status_flags.h"
-#include "services/network/public/cpp/resource_response.h"
+#include "services/network/public/mojom/fetch_api.mojom-shared.h"
 #include "services/network/public/mojom/url_loader.mojom.h"
+#include "services/network/public/mojom/url_response_head.mojom.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "url/origin.h"
 
 namespace download {
@@ -52,15 +55,21 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadResponseHandler
       const DownloadUrlParameters::RequestHeadersType& request_headers,
       const std::string& request_origin,
       DownloadSource download_source,
-      bool ignore_content_length_mismatch,
+      bool require_safety_checks,
       std::vector<GURL> url_chain,
       bool is_background_mode);
+
+  DownloadResponseHandler(const DownloadResponseHandler&) = delete;
+  DownloadResponseHandler& operator=(const DownloadResponseHandler&) = delete;
+
   ~DownloadResponseHandler() override;
 
   // network::mojom::URLLoaderClient
-  void OnReceiveResponse(const network::ResourceResponseHead& head) override;
+  void OnReceiveEarlyHints(network::mojom::EarlyHintsPtr early_hints) override;
+  void OnReceiveResponse(network::mojom::URLResponseHeadPtr head,
+                         mojo::ScopedDataPipeConsumerHandle body) override;
   void OnReceiveRedirect(const net::RedirectInfo& redirect_info,
-                         const network::ResourceResponseHead& head) override;
+                         network::mojom::URLResponseHeadPtr head) override;
   void OnUploadProgress(int64_t current_position,
                         int64_t total_size,
                         OnUploadProgressCallback callback) override;
@@ -72,12 +81,12 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadResponseHandler
 
  private:
   std::unique_ptr<DownloadCreateInfo> CreateDownloadCreateInfo(
-      const network::ResourceResponseHead& head);
+      const network::mojom::URLResponseHead& head);
 
   // Helper method that is called when response is received.
   void OnResponseStarted(mojom::DownloadStreamHandlePtr stream_handle);
 
-  Delegate* const delegate_;
+  const raw_ptr<Delegate> delegate_;
 
   std::unique_ptr<DownloadCreateInfo> create_info_;
 
@@ -88,7 +97,7 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadResponseHandler
   std::vector<GURL> url_chain_;
   std::string method_;
   GURL referrer_;
-  net::URLRequest::ReferrerPolicy referrer_policy_;
+  net::ReferrerPolicy referrer_policy_;
   bool is_transient_;
   bool fetch_error_body_;
   network::mojom::RedirectMode cross_origin_redirects_;
@@ -97,20 +106,22 @@ class COMPONENTS_DOWNLOAD_EXPORT DownloadResponseHandler
   std::string request_origin_;
   DownloadSource download_source_;
   net::CertStatus cert_status_;
-  bool ignore_content_length_mismatch_;
-  base::Optional<url::Origin> request_initiator_;
+  bool has_strong_validators_;
+  absl::optional<url::Origin> request_initiator_;
+  ::network::mojom::CredentialsMode credentials_mode_;
+  absl::optional<net::IsolationInfo> isolation_info_;
   bool is_partial_request_;
   bool completed_;
+  bool require_safety_checks_;
 
   // The abort reason if this class decides to block the download.
   DownloadInterruptReason abort_reason_;
 
-  // Mojo interface ptr to send the completion status to the download sink.
-  mojom::DownloadStreamClientPtr client_ptr_;
+  // Mojo interface remote to send the completion status to the download sink.
+  mojo::Remote<mojom::DownloadStreamClient> client_remote_;
 
   // Whether the download is running in background mode.
   bool is_background_mode_;
-  DISALLOW_COPY_AND_ASSIGN(DownloadResponseHandler);
 };
 
 }  // namespace download
